from __future__ import division
import math, sys
from collections import OrderedDict
import sympy as sp
import numpy as np
import numpy.matlib as ml
import numpy.linalg as la
import dk_templates
from codelib import *
import dk_simulator

def solver_set_defaults(solver_dict=None):
    if solver_dict is None:
        solver_dict = {}
    else:
        solver_dict = solver_dict.copy()
    solver_dict.setdefault("method", "hybr")
    solver_dict.setdefault("factor", 1.e2)
    solver_dict.setdefault("xtol", math.sqrt(sys.float_info.epsilon))
    solver_dict.setdefault("maxfev", 2000)
    solver_dict.setdefault("max_homotopy_iter", 100)
    return solver_dict


class Structure(object):

    def __init__(self, name):
        self.name = name
        self.members = []
        self.nonlin_info = False
        self.minmax_info = None

    def add(self, m):
        self.members.append(m)

    def add_nonlin_info(self):
        self.nonlin_info = True;

    def add_minmax_info(self, rows):
        self.minmax_info = rows

    def get_initializer(self, kw):
        s = dict((m.name, i) for i, m in enumerate(v for v in self.members if v.pointer))
        n = len(s)
        if self.nonlin_info:
            s['info'] = n
            s['nfev'] = n+1
            s['fnorm'] = n+2
            n += 3
        if self.minmax_info:
            s['p_val'] = n
            n += 1
        l = [None]*n
        s2 = set()
        for k, v in kw.items():
            try:
                i = s[k]
            except KeyError:
                raise ValueError("unknown struct member: %s" % k)
            if isinstance(v, VariableAccess):
                l[i] = str(v.address_of())
            else:
                l[i] = v
            s2.add(k)
        missing = set(s) - s2
        if missing:
            raise ValueError("no structure initialization for: %s" % ", ".join(missing))
        if not l:
            return ""
        return "("+", ".join(l)+")"

    def generate_lines(self):
        arglist = [m.generate("_") for m in self.members if m.pointer]
        initlist = []
        for m in self.members:
            if m.pointer:
                initlist.append("%s(%s_)" % (m.name, m.name))
            else:
                initlist.append("%s()" % m.name)
        l = []
        l.append("struct %s {\n" % self.name)
        l += ["    %s;\n" % m for m in self.members]
        if self.nonlin_info:
            l.append("    int *info;\n")
            l.append("    int *nfev;\n")
            l.append("    creal *fnorm;\n")
            initlist += ["info(info_)", "nfev(nfev_)", "fnorm(fnorm_)"]
            arglist += ["int *info_", "int *nfev_", "creal *fnorm_"]
        if self.minmax_info:
            p_val = MatrixDeclaration('p_val', rows=self.minmax_info, cols=1, pointer=True, array=True)
            l.append("    %s;\n" % p_val)
            initlist.append("p_val(p_val_)")
            arglist.append(p_val.generate("_"))
        if initlist:
            l.append("    inline %(name)s(%(args)s): %(init)s {}\n" % dict(
                    name = self.name,
                    args = ", ".join(arglist),
                    init = ", ".join(initlist),
                    ))
        l.append("};")
        return l

    def __str__(self):
        return "".join(self.generate_lines())


class NonlinFunction(object):

    def __init__(self, global_ns, neq, on):
        self.global_ns = global_ns
        self.neq = neq
        self.have_constant_matrices = (neq.eq.np == 0)
        self.locals = Namespace()
        self.input_slice = on.input_slice
        self.output_slice = on.output_slice

    def expr_list(self, v):
        par_v = make_symbol_vector('(*par.v)', self.neq.nn)
        l = []
        f = self.neq.eq.f
        off = 0
        if self.neq.v_slice:
            off = self.neq.v_slice.start
            f = f[self.neq.v_slice]
        for expr, vl, base in f:
            for var, idx in zip(vl, base):
                if idx >= off:
                    vv = v[idx-off]
                else:
                    vv = par_v[idx]
                expr = expr.subs(var, vv)
            l.append(expr)
        return l

    def generate(self, template):
        if self.neq.v_slice:
            iblockV = self.input_slice
            oblockV = self.output_slice
            blockM = (self.neq.v_slice, self.neq.v_slice)
            start = self.output_slice.start
        else:
            iblockV = oblockV = None
            blockM = None
            start = 0
        v = make_symbol_vector('v', self.neq.nn)
        i = VectorAccess('i', param=True)
        l = []
        l += expr_list_to_ccode(str(i), self.expr_list(v), '(%d)', start)
        mv = MatrixDefinition('mv', rows=self.neq.nn, cols=1)
        Mfvec = MatrixDefinition('Mfvec', value='fvec', rows=self.neq.nn, cols=1)
        self.locals.add(mv)
        self.locals.add(Mfvec)
        l += self.locals.generate_lines()
        CZ = self.neq.eq.CZ[self.neq.v_slice]
        K = self.neq.eq.K
        if self.have_constant_matrices:
            K = K[blockM]
        l.append("mv << %s;\n" % ", ".join(
            ['v[%d]' % i if z else '0' for i, z in enumerate(CZ)]))
        if not matrix_is_identity(K):
            if self.have_constant_matrices:
                self.global_ns.add(MatrixDefinition('K', K))
                K = MatrixAccess('K')
            else:
                K = MatrixAccess('K', param=True, block=blockM, pointer=False)
            l.append("Mfvec = %(p)s + %(K)s * %(i)s - %(mv)s;\n" % dict(
                p = VectorAccess('p', param=True, block=iblockV),
                K = K,
                i = VectorAccess('i', param=True, block=oblockV),
                mv = VectorAccess('mv'),
                ))
        else:
            l.append("Mfvec = %(p)s + %(i)s - %(mv)s;" % dict(
                p = VectorAccess('p', param=True, block=iblockV),
                i = VectorAccess('i', param=True, block=oblockV),
                mv = VectorAccess('mv'),
                ))
        return template.render(dict(expression=join_with_indent(l)))


class NonlinFunctionCC(object):

    def __init__(self, global_ns, neq, extra_sources):
        self.global_ns = global_ns
        self.neq = neq
        self.extra_sources = extra_sources

    def generate(self, template):
        neq = self.neq
        have_constant_matrices = (neq.eq.np == 0)
        end = neq.cc_slice.start
        cc = neq.cc_slice.stop - neq.cc_slice.start
        p0_slice = slice(neq.p_slice.start, neq.p_slice.stop-cc)
        p1_slice = slice(neq.p_slice.stop-cc, neq.p_slice.stop)
        i0_slice = slice(neq.i_slice.start, neq.i_slice.stop-cc)
        i1_slice = slice(neq.i_slice.stop-cc, neq.i_slice.stop)
        M0 = (slice(0, end), slice(end, neq.nn))
        M1 = (slice(end, neq.nn), slice(0, neq.nn))
        loc = Namespace()
        l = []
        loc.add(MatrixDefinition('Mv', value='v', rows=neq.nn-end, cols=1, aligned=False, const=True))
        if self.extra_sources:
            loc.add(MatrixDefinition('pt', rows=p0_slice.stop-p0_slice.start, cols=1, array=True))
            pt = VectorAccess('pt')
        else:
            loc.add(MatrixDefinition('pt', rows=neq.nn, cols=1))
            pt = VectorAccess('pt', block=p0_slice)
        l += loc.generate_lines()
        if have_constant_matrices:
            #self.global_ns.add(MatrixDefinition('Ku', neq.eq.K[M0]))
            self.global_ns.add(MatrixDefinition('Ku', neq.Ku))
            Ku = MatrixAccess('Ku')
        else:
            Ku = MatrixAccess('K', param=True, block=M0, pointer=False)
        l.append("%s = %s + %s * %s;\n" % (
            pt,
            VectorAccess('p', param=True, block=p0_slice),
            Ku,
            VectorAccess('Mv')))
        if self.extra_sources:
            shape_transform = self.extra_sources.get("shape_transform", ())
            for mat in shape_transform:
                self.global_ns.add(mat)
            if shape_transform:
                def nth(j):
                    idx = []
                    for bl in neq.subblocks:
                        ln = bl.v_slice.stop - bl.v_slice.start
                        idx.append(j)
                        j += ln
                    return idx
                def spliced(n):
                    for a in range(n):
                        yield a
                        yield a+n
                nn2 = end//2
                loc.add(MatrixDeclaration('PP1', rows=nn2, cols=1, array=True))
                l += loc.generate_lines()
                l.append("PP1 << %s;\n" % ", ".join(["pt(%d)" % i for i in nth(1)]))
                loc.add(MatrixDeclaration('PP0', rows=nn2, cols=1, array=True))
                l += loc.generate_lines()
                l.append("PP0 << %s;\n" % ", ".join(["pt(%d)" % i for i in nth(0)]))
                l += loc.generate_lines()
                l.append("pt.head<%d>() = (Spm1 * PP1 + Ssm1) * PP1 + Sam1 + ((Spm2 * PP1 + Ssm2) * PP1 + Sam2) * PP0;\n" % nn2)
                loc.add(MatrixDeclaration('res', rows=end, cols=1, array=True))
                l += loc.generate_lines()
                inp = ["&pt(%d)" % (i//2) for i in range(end)]
                outp = ["&res(%d)" % i for i in spliced(nn2)]
            else:
                inp = []
                outp = []
                for bl in neq.subblocks:
                    o = VectorAccess('i', param=True)
                    for j in range(bl.i_slice.start, bl.i_slice.stop):
                        inp.append("&pt(%d)" % bl.p_slice.start)
                        outp.append("&%s(%d)" % (o, j))
            jj = 0
            tables = self.extra_sources["tables"]
            for bl in neq.subblocks:
                for j, kn in enumerate(tables[bl.namespace].knot_data):
                    reorder = False
                    unused = False
                    ll = []
                    for i, v in enumerate(kn):
                        if not v.used():
                            unused = True
                        else:
                            ll.append(i)
                            if unused:
                                reorder = True
                    if reorder:
                        l.append("{ Array<creal, %d, 1> pt2; pt2 << %s;\n" % (len(ll), ", ".join(["pt(%d)" % (bl.p_slice.start+i) for i in ll])))
                        inpt = "&pt2(0)"
                    else:
                        inpt = inp[jj]
                    fu = "splev"
                    if kn[0].tp == 'pp':
                        fu = "splev_pp"
                    l.append("splinedata<AmpData::%s::maptype>::%s<%s>(&AmpData::%s::sd.sc[%d], %s, %s);\n"
                             % (bl.namespace, fu, ",".join([str(v.get_order()) for v in kn if v.used()]), bl.namespace, j, inpt, outp[jj]))
                    if reorder:
                        l.append("}\n")
                    jj += 1
            if shape_transform:
                l.append("pt.head<%d>() = ((Spm0 * PP1 + Ssm0) * PP1 + Sam0) * res.head<%d>();\n" % (nn2,nn2))
                l.append("pt.tail<%d>() = ((Spm0 * PP1 + Ssm0) * PP1 + Sam0) * res.tail<%d>();\n" % (nn2,nn2))
                l.append("%s << %s;\n" % (VectorAccess('i', param=True, block=i0_slice),
                                          ", ".join(["pt(%d)" % v for v in spliced(nn2)])))
        else:
            loc.add(MatrixDefinition('pp', rows=neq.nn, cols=1, pointer=True))
            l += loc.generate_lines()
            l.append("%s = %s;\n" % (
                MatrixAccess('pp', pointer=True).address_of(),
                MatrixAccess('p', param=True).address_of()))
            restore = "%s = %s;\n" % (
                MatrixAccess('p', param=True).address_of(),
                MatrixAccess('pp', pointer=True).address_of())
            l.append("%s = %s;\n" % (
                MatrixAccess('p', param=True).address_of(),
                MatrixAccess('pt').address_of()))
            l.append("int ret;\n")
            for bl in neq.subblocks:
                l.append("ret = %s::nonlin_solve(par);\n" % bl.namespace)
                l.append("if (ret != 0) {\n")
                l.append("    "+restore)
                l.append("    return 1;\n")
                l.append("};\n")
            l.append(restore)
        l.append("%s = %s;\n" % (VectorAccess('i', param=True, block=i1_slice), VectorAccess('Mv')))
        if have_constant_matrices:
            if M1:
                self.global_ns.add(MatrixDefinition('Kl', neq.Kl))
                Kl = MatrixAccess('Kl')
            else:
                Kl = MatrixAccess('K', block=M1)
        else:
            Kl = MatrixAccess('K', param=True, block=M1, pointer=False)
        Mfvec = MatrixDefinition('Mfvec', value='fvec', rows=neq.nn-end, cols=1)
        loc.add(Mfvec)
        l += loc.generate_lines()
        l.append("Mfvec = %(p)s + %(K)s * %(i)s;\n" % dict(
            p = VectorAccess('p', param=True, block=p1_slice),
            K = Kl,
            i = VectorAccess('i', param=True, block=neq.i_slice),
            ))
        return template.render(dict(expression=join_with_indent(l)))


class NonlinSolver(object):

    def __init__(self, base, glob, neq, solver_dict):
        self.base = base
        self.neq = neq
        self.solver_dict = solver_dict
        self.have_constant_matrices = (neq.eq.np == 0)
        self.global_ns = glob
        self.local_ns = Namespace()
        self.mp_is_ident = matrix_is_identity(self.neq.U)
        self.mpc_is_zero = matrix_is_zero(self.neq.Hc)
        if self.mp_is_ident and self.mpc_is_zero:
            self.input_slice = self.neq.p_slice
        else:
            self.input_slice = slice(0, self.neq.nn)
        self.mi_is_ident = matrix_is_identity(self.neq.Mi)
        if self.mi_is_ident:
            self.output_slice = self.neq.i_slice
        else:
            self.output_slice = slice(0, self.neq.nn)
        self.base["nn"] = self.neq.nn
        self.base["nni"] = self.neq.nni
        self.base["nno"] = self.neq.nno

    def p_transform(self):
        l = []
        if self.mp_is_ident and self.mpc_is_zero:
            return l
        par = not self.have_constant_matrices
        par_p = VectorAccess('p', param=True, block=self.neq.p_slice)
        if not self.mp_is_ident:
            if not par:
                self.global_ns.add(MatrixDefinition('Mp', value=self.neq.U))
            s = "%s * %s" % (MatrixAccess('Mp', param=par), par_p)
        else:
            s = "%s" % par_p
        if not self.mpc_is_zero:
            if not par:
                self.global_ns.add(MatrixDefinition('Mpc', value=self.neq.Hc))
            s += " + %s" % VectorAccess('Mpc', param=par, pointer=False)
        g_nn = self.neq.eq.nonlin.nn
        p_old_def = MatrixDefinition("p_old", rows=g_nn, cols=1, pointer=True)
        self.local_ns.add(p_old_def)
        p_old = VectorAccess(p_old_def)
        p2_def = MatrixDefinition("p2", rows=g_nn, cols=1)
        self.local_ns.add(p2_def)
        p2 = VectorAccess(p2_def, block=self.input_slice)
        l += self.local_ns.generate_lines()
        l.append("%s = %s;\n" % (p2, s))
        l.append("p_old = %s;\n" % par_p.address_of())
        l.append("%s = &p2;\n" % par_p.address_of())
        self.cleanup.append("%s = p_old;\n" % par_p.address_of())
        return l

    def i_transform(self):
        l = []
        if self.mi_is_ident or self.neq.nno == 0:
            return l
        g_nn = self.neq.eq.nonlin.nn
        mi_def = MatrixDefinition("mi", rows=g_nn, cols=1)
        mi = VectorAccess(mi_def, block=self.output_slice)
        i_old_def = MatrixDefinition("i_old", rows=g_nn, cols=1, pointer=True)
        i_old = VectorAccess(i_old_def)
        par_i = VectorAccess('i', param=True, block=self.neq.i_slice)
        self.global_ns.add(MatrixDefinition("Mi", self.neq.Mi))
        self.local_ns.add(mi_def)
        self.local_ns.add(i_old_def)
        self.setup += self.local_ns.generate_lines()
        self.setup.append("i_old = %s;\n" % VectorAccess('i', param=True).address_of())
        self.setup.append("%s = &mi;\n" % VectorAccess('i', param=True).address_of())
        l.append("%s = i_old;\n" % VectorAccess('i', param=True).address_of())
        par = not self.have_constant_matrices
        l.append("%(io)s = %(Mi)s * %(ii)s;" % dict(
                io = par_i,
                Mi = MatrixAccess('Mi', param=par, pointer=False),
                ii = mi,
                ))
        return l

    def make_var_v_ref(self):
        start = 0
        if self.neq.v_slice:
            start = self.neq.v_slice.start
        return "&%s(%s)" % (VectorAccess("v", param=True), start)

    def generate(self, template):
        d = self.base.copy()
        self.setup = []
        self.cleanup = []
        d["i_transform"] = join_with_indent(self.i_transform())
        d["p_transform"] = join_with_indent(self.p_transform())
        d["setup"] = join_with_indent(self.setup)
        d["cleanup"] = join_with_indent(self.cleanup)
        d["var_v_ref"] = self.make_var_v_ref()
        d["store_p"] = "%s = %s;\n" % (
            VectorAccess('p_val', param=True, block=self.neq.p_slice),
            VectorAccess('p', param=True, block=self.neq.p_slice),
            )
        return template.render(d)

class NonlinSolverCC(NonlinSolver):

    def __init__(self, base, glob, neq, solver_dict):
        NonlinSolver.__init__(self, base, glob, neq, solver_dict)
        self.input_slice = self.neq.p_slice
        self.mp_is_ident = True
        self.mi_is_ident = True

    def make_var_v_ref(self):
        return "&%s(%s)" % (VectorAccess("v", param=True), self.neq.cc_slice.start)

    def generate(self, template):
        self.base = self.base.copy()
        self.base["nn"] = self.base["nni"] = self.base["nno"] = self.neq.cc_slice.stop - self.neq.cc_slice.start
        return NonlinSolver.generate(self, template)


class NonlinCode(object):

    method_templates = {
        'hybr':   (dk_templates.c_template_nonlin_func_hybrCC,
                   dk_templates.c_template_nonlin_solver_hybrCC),
        'lm':     (dk_templates.c_template_nonlin_func_lm,
                   dk_templates.c_template_nonlin_solver_lm),
        'hybrCC': (dk_templates.c_template_nonlin_func_hybrCC,
                   dk_templates.c_template_nonlin_solver_hybrCC),
        }

    def __init__(self, struct, neq, solver_dict, extra_sources):
        self.struct = struct
        self.neq = neq
        self.solver_dict = solver_dict
        self.extra_sources = extra_sources

    def setup(self, d):
        neq = self.neq
        base = {}
        base["v0_guess"] = d["v0_guess"]
        base["dev_interface"] = d["dev_interface"]
        g_nonlin = neq.eq.nonlin
        base["extern_nonlin"] = not neq.subblocks or neq is g_nonlin
        base["g_nn"] = g_nonlin.nn
        base["g_nni"] = g_nonlin.nni
        base["g_nno"] = g_nonlin.nno
        base["npl"] = neq.eq.get_npl()
        if neq.nn != g_nonlin.nn:
            nn = base["nn"] = neq.nn
            nni = base["nni"] = neq.nni
            nno = base["nno"] = neq.nno
            self.blockV = neq.v_slice
            self.pblockV = neq.p_slice
            base["pblockV"] = VectorAccess.block_expr(neq.p_slice)
            base["iblockV"] = VectorAccess.block_expr(neq.i_slice)
        else:
            base["nn"] = nn = neq.nn
            base["nni"] = nni = neq.nni
            base["nno"] = nno = neq.nno
            self.blockV = None
            if nn != nni:
                self.pblockV = slice(0, nni)
                base["pblockV"] = VectorAccess.block_expr([0, nni])
            else:
                self.pblockV = None
                base["pblockV"] = ""
            if nn != nno:
                base["iblockV"] = VectorAccess.block_expr([0, nno])
            else:
                base["iblockV"] = ""
        if self.solver_dict:
            for k, v in self.solver_dict.items():
                base["solver_"+k] = v
        ini = dict(
            p = MatrixAccess('mp'),
            i = MatrixAccess('mi'),
            v = MatrixAccess('Mv'),
            info = 'info',
            fnorm = 'fnorm',
            nfev = 'nfev',
            p_val = MatrixAccess('p_val'),
            )
        base["nonlin_mat_list"] = self.struct.get_initializer(ini)
        self.glob = Namespace()
        base["namespace"] = neq.namespace
        base["global_data_def"] = self.glob
        return base

    def generate(self, d):
        base = self.setup(d)
        d = base.copy()
        neq = self.neq
        func_t, solv_t = self.method_templates[self.solver_dict["method"]]
        if isinstance(neq, dk_simulator.PartitionedNonlinEquations):
            of = NonlinFunctionCC(self.glob, neq, self.extra_sources)
            on = NonlinSolverCC(base, self.glob, neq, self.solver_dict)
        else:
            on = NonlinSolver(base, self.glob, neq, self.solver_dict)
            of = NonlinFunction(self.glob, neq, on)
        if func_t is not None:
            d["fcn_def"] = of.generate(func_t)
        d["nonlin_def"] = on.generate(solv_t)
        d["par_p"] = VectorAccess("p", param=True, block=self.pblockV)
        d["par_v"] = VectorAccess("v", param=True, block=self.blockV)
        return dk_templates.c_template_nonlin_solver.render(d)


class NonlinChained(NonlinCode):

    def __init__(self, s, neq):
        NonlinCode.__init__(self, s, neq, None, None)

    def generate(self, d):
        d = self.setup(d)
        self.glob.add(MatrixDefinition('K', self.neq.eq.K))
        template = dk_templates.c_template_nonlin_chained
        l = []
        l.append("%s = %s;\n" % (
            VectorAccess('p_val', param=True, block=self.neq.p_slice),
            VectorAccess('p', param=True, block=self.neq.p_slice),
            ))
        g_nn = self.neq.eq.nonlin.nn
        p_old_def = MatrixDefinition("p_old", rows=g_nn, cols=1, pointer=True)
        local_ns = Namespace()
        local_ns.add(p_old_def)
        p_old = VectorAccess(p_old_def)
        p2_def = MatrixDefinition("p2", rows=g_nn, cols=1)
        local_ns.add(p2_def)
        p2 = VectorAccess(p2_def, block=self.neq.p_slice)
        l += local_ns.generate_lines()
        l.append("int ret = 0;\n")
        l.append("p2 = *par.p;\n")
        l.append("p_old = par.p;\n")
        l.append("par.p = &p2;\n")
        for nlin in self.neq.subblocks:
            st = nlin.p_slice.start
            if st:
                l.append("p2.segment<%(ln)d>(%(st)d) += K.block<%(ln)d,%(st)d>(%(st)d,0) * (*par.i).head<%(st)d>();\n"
                         % dict(ln=nlin.p_slice.stop-st, st=st))
            l.append("ret = %s::nonlin_solve(par);\n" % nlin.namespace)
            l.append("if (ret != 0) {\n")
            l.append("    par.p = p_old;\n")
            l.append("    return ret;\n")
            l.append("}\n")
        l.append("par.p = p_old;\n")
        l.append("return 0;\n")
        d["chained_code"] = join_with_indent(l)
        return template.render(d)


class TableCode(object):

    def __init__(self, struct, neq, extra_sources):
        self.struct = struct
        self.neq = neq
        self.extra_sources = extra_sources

    def add(self, d):
        neq = self.neq
        base = {}
        base["dev_interface"] = d["dev_interface"]
        g_nonlin = neq.eq.nonlin
        base["g_nn"] = g_nonlin.nn
        base["g_nni"] = g_nonlin.nni
        base["g_nno"] = g_nonlin.nno
        base["npl"] = neq.eq.get_npl()
        nn = base["nn"] = neq.nn
        nni = base["nni"] = neq.nni
        nno = base["nno"] = neq.nno
        if neq.v_slice:
            self.blockV = neq.v_slice
            self.pblockV = neq.p_slice
            self.iblockV = neq.i_slice
            base["blockV"] = VectorAccess.block_expr(neq.v_slice)
            base["pblockV"] = VectorAccess.block_expr(neq.p_slice)
            base["iblockV"] = VectorAccess.block_expr(neq.i_slice)
        else:
            self.blockV = None
            base["blockV"] = ""
            if nn != nni:
                self.pblockV = slice(0, nni)
                base["pblockV"] = VectorAccess.block_expr([0, nni])
            else:
                self.pblockV = None
                base["pblockV"] = ""
            if nn != nno:
                base["iblockV"] = VectorAccess.block_expr([0, nno])
            else:
                base["iblockV"] = ""
        ini = dict(
            p = MatrixAccess('mp'),
            i = MatrixAccess('mi'),
            v = MatrixAccess('Mv'),
            info = '&g_info',
            fnorm = 'fnorm',
            nfev = '&g_nfev',
            p_val = MatrixAccess('p_val'),
            )
        base["nonlin_mat_list"] = self.struct.get_initializer(ini)
        self.glob = Namespace()
        d = base.copy()
        d["namespace"] = neq.namespace
        d["global_data_def"] = self.glob
        d["par_p"] = VectorAccess("p", param=True, block=self.pblockV)
        d["par_v"] = VectorAccess("v", param=True, block=self.iblockV)
        return d

    def generate(self, d):
        d = self.add(d)
        neq = self.neq
        tables = self.extra_sources["tables"]
        l = []
        l.append("real t[AmpData::%(namespace)s::sd.m];\n" % d)
        l.append("real m[%(nni)d+%(npl)d];\n" % d)
        l.append("Map<Matrix<real, %(nni)d+%(npl)d, 1> >mp(m);\n" % d)
        l.append("mp << last_pot.cast<real>(), (*par.p)%(pblockV)s.cast<real>();\n" % d)
        for j, kn in enumerate(tables[neq.namespace].knot_data):
            reorder = False
            unused = False
            fu = "splev"
            ll = []
            for i, v in enumerate(kn):
                if not v.used():
                    unused = True
                else:
                    ll.append(i)
                    if unused:
                        reorder = True
                    if kn[i].tp == 'pp':
                        fu = "splev_pp"
            if reorder:
                l.append("{ Array<creal, %d, 1> pt2; pt2 << %s;\n" % (len(ll), ", ".join(["mp(%d)" % i for i in ll])))
                inpt = "&pt2(0)"
            else:
                inpt = "m"
            l.append("splinedata<AmpData::%s::maptype>::%s<%s>(&AmpData::%s::sd.sc[%d], %s, &t[%d]);\n"
                     % (neq.namespace, fu, ",".join([str(v.get_order()) for v in kn if v.used()]), neq.namespace, j, inpt, j))
            if reorder:
                l.append("}\n")
        l.append("(*par.i)%(iblockV)s = Map<Matrix<real, %(nno)d, 1> >(t).cast<creal>();\n" % d)
        d["call"] = join_with_indent(l)
        return dk_templates.c_template_table.render(d)


class UpdateMatrix(object):

    def __init__(self, glob, eq, pot, pot_list, pot_func, Pv):
        self.glob = glob
        self.eq = eq
        self.pot = pot
        self.pot_func = pot_func
        self.pot_list = pot_list
        self.Pv = Pv

    def trans_line(self, res, var, var_data, t, u, u_data):
        if not isinstance(res, VariableAccess):
            res = MatrixAccess(res)
        if not isinstance(var, VariableAccess):
            var = MatrixAccess(var)
        self.glob.add(MatrixDefinition(var, var_data))
        self.glob.add(MatrixDefinition(res, rows=var_data.shape[0], cols=var_data.shape[1]))
        s = "%s = %s" % (res, var)
        if t is None:
            return s + ";\n"
        if u_data is not None:
            self.glob.add(MatrixDefinition(u, u_data))
        return s + " - %s * %s;\n" % (t, u)

    def generate(self, struct):
        eq = self.eq
        d = {}
        pot = make_symbol_vector('pot', eq.np)
        l = []
        for (a, f), p in zip(self.pot_func, self.Pv):
            s = str(a)
            try:
                i = self.pot_list.index(s)
            except ValueError:
                self.pot_list.append(s)
                i = len(self.pot_list)-1
            expr = f.subs(a, pot[i]) * p
            l.append(expr)
        d["pot_vars"] = ",".join(['"%s"' % v for v in self.pot_list])
        d["pot"] = ",".join([str(self.pot.get(v,0.5)) for v in self.pot_list])
        nx = eq.nx
        no = eq.no
        np = eq.np
        nn = eq.nn
        loc = Namespace()
        loc.add(MatrixDeclaration("Rv", rows=np, cols=1))
        lines = []
        lines += loc.generate_lines()
        lines += expr_list_to_ccode('Rv', l, '(%d)')
        self.glob.add(MatrixDefinition("Q", eq.Q))
        loc.add(MatrixDeclaration("Qi", rows=np, cols=np))
        lines += loc.generate_lines()
        lines.append("Qi = (%s + Matrix<creal, %d, %d>(Rv.asDiagonal())).inverse();\n" % (MatrixAccess('Q'), np, np))
        if eq.nx:
            if matrix_is_identity(eq.Uxl):
                t = "Qi"
            elif matrix_is_zero(eq.Uxl):
                t = None
            else:
                loc.add(MatrixDeclaration("Tx", rows=nx, cols=np))
                lines += loc.generate_lines()
                self.glob.add(MatrixDefinition("Uxl", eq.Uxl))
                lines.append("Tx = %s * Qi;\n" % MatrixAccess('Uxl'))
                t = "Tx"
            Mx_mat = eq.get_Mx()
            m = MatrixDeclaration('Mx', rows=Mx_mat.shape[0], cols=Mx_mat.shape[1])
            struct.add(m)
            Mx = MatrixAccess(m, param=True)
            lines.append(self.trans_line(Mx, 'Mx', Mx_mat, t, 'UR', eq.get_UR()))
            m = MatrixDeclaration('Mxc', rows=eq.Bc.shape[0], cols=1)
            struct.add(m)
            Mxc = VectorAccess(m, param=True)
            lines.append(self.trans_line(Mxc, 'Mxc', eq.Bc, t, 'Ucv', eq.Ucv.T))
        if matrix_is_identity(eq.Uo):
            t  = "Qi"
        elif matrix_is_zero(eq.Uo):
            t = None
        else:
            loc.add(MatrixDeclaration("To", rows=no, cols=np))
            lines += loc.generate_lines()
            self.glob.add(MatrixDefinition('Uo', eq.Uo))
            lines.append("To = %s * Qi;\n" % MatrixAccess('Uo'))
            t = "To"
        Mo_mat = eq.get_Mo()
        m = MatrixDeclaration('Mo', rows=Mo_mat.shape[0], cols=Mo_mat.shape[1])
        struct.add(m)
        Mo = MatrixAccess(m, param=True)
        lines.append(self.trans_line(Mo, 'Mo', Mo_mat, t, 'UR', eq.get_UR()))
        m = MatrixDeclaration('Moc', rows=eq.Ec.shape[0], cols=1)
        struct.add(m)
        Moc = VectorAccess(m, param=True)
        lines.append(self.trans_line(Moc, 'Moc', eq.Ec, t, 'Ucv', eq.Ucv.T))
        if eq.nonlin and eq.nonlin.nni:
            if matrix_is_identity(eq.Unl):
                t = "Qi"
            elif  matrix_is_zero(eq.Unl):
                t = None
            else:
                loc.add(MatrixDeclaration("Tp", rows=nn, cols=np))
                lines += loc.generate_lines()
                self.glob.add(MatrixDefinition("Unl", eq.Unl))
                lines.append("Tp = %s * Qi;\n" % MatrixAccess('Unl'))
                t = "Tp"
            Mp_mat = eq.get_Mp()
            struct.add(MatrixDeclaration('Mp', rows=Mp_mat.shape[0], cols=Mp_mat.shape[1]))
            lines.append(self.trans_line(MatrixAccess('Mp', param=True, pointer=False), 'Mp', Mp_mat, t, 'UR.block<%d, %d>(0, 0)' % (eq.np, eq.mp_cols), None))
            struct.add(MatrixDeclaration('Mpc', rows=eq.nonlin.Hc.shape[0], cols=1))
            lines.append(self.trans_line(VectorAccess('Mpc', param=True, pointer=False), 'Mpc', eq.nonlin.Hc, t, 'Ucv', eq.Ucv.T))
            struct.add(MatrixDeclaration('K', rows=eq.K.shape[0], cols=eq.K.shape[1]))
            mp_cols = eq.get_mp_cols()
            lines.append(self.trans_line(MatrixAccess('K', param=True, pointer=False), 'K', eq.K, t, 'UR.block<%d, %d>(0, %d)' % (eq.np, eq.get_mx_cols()-mp_cols, mp_cols), None))
        d["update_pot"] = join_with_indent(lines)
        return d


class LinearCode(object):

    def __init__(self, glob, eq):
        self.glob = glob
        self.eq = eq

    def generate(self):
        eq = self.eq
        param = bool(eq.np)
        Mp = MatrixAccess('Mp', param=param, pointer=False)
        Mo = MatrixAccess('Mo', param=param, pointer=False)
        Moc = VectorAccess('Moc', param=param, pointer=False)
        Mx = MatrixAccess('Mx', param=param, pointer=False)
        Mxc = VectorAccess('Mxc', param=param, pointer=False)
        d = {}
        if eq.nonlin and eq.nn != eq.nonlin.nni:
            pblock = slice(0, eq.nonlin.nni)
        else:
            pblock = None
        d["m_cols"] = eq.get_mx_cols()
        d["gen_mp"] = gen_linear_combination(
            self.glob, VectorAccess('mp',block=pblock), 'dp', Mp, eq.get_Mp())
        d["gen_xn"] = gen_linear_combination(
            self.glob, 'xn', 'd', Mx, eq.get_Mx(), Mxc, eq.Bc)
        d["gen_xo"] = gen_linear_combination(
            self.glob, 'xo', 'd', Mo, eq.get_Mo(), Moc, eq.Ec)
        d["gen_xo_float"] = gen_linear_combination(
            self.glob, 'xo', 'd', Mo, eq.get_Mo(), cast="float")
        return d


class NonlinEq(object):

    def __init__(self, nn, nni, nno, np, npl, K, CZ, f, U, Hc, Mi, Kn):
        self.nn = nn
        self.nni = nni
        self.nno = nno
        self.np =  np
        self.npl =  npl
        self.K = K
        self.CZ = CZ
        self.f = f
        self.U = U
        self.Hc = Hc
        self.Mi = Mi
        self.Kn = Kn


class LV2_Port_List(object):

    def __init__(self, pot_attr, pot, eq):
        self.pot_attr = pot_attr
        self.pot = pot
        self.ni = eq.ni
        self.no = eq.no

    def port_count(self):
        return self.ni + self.no + len(self.pot_attr)

    def __len__(self):
        return self.ni + self.no + len(self.pot_attr)

    def __iter__(self):
        idx = 0
        max_idx = len(self) - 1
        for i in range(self.ni):
            yield dict(
                type_list="lv2:AudioPort , lv2:InputPort",
                index = idx,
                symbol = "in%d" % i,
                name = "In%d" % i,
                control_index = -1,
                separator = "," if idx == max_idx else "",
                )
            idx += 1
        for i in range(self.no):
            yield dict(
                type_list="lv2:AudioPort , lv2:OutputPort",
                index = idx,
                symbol = "out%d" % i,
                name = "Out%d" % i,
                control_index = -1,
                separator = "," if idx == max_idx else "",
                )
            idx += 1
        for i, row in enumerate(self.pot_attr):
            var = row[0]
            name = row[1]
            yield dict(
                type_list="lv2:InputPort , lv2:ControlPort",
                index = idx,
                symbol = var,
                name = name,
                default = self.pot.get(var, 0.5),
                minimum = 0.0,
                maximum = 1.0,
                control_index = i,
                separator = "," if idx == max_idx else "",
                )
            idx += 1


class CodeGenerator(object):

    def __init__(self, eq, solver_dict, solver_params, pot, pot_list, pot_func, pot_attr, Pv, extra_sources):
        self.eq = eq
        self.solver_dict = solver_set_defaults(solver_dict)
        self.solver_params = solver_params
        self.pot = pot
        self.pot_list = pot_list
        self.pot_func = pot_func
        self.pot_attr = pot_attr
        self.Pv = Pv
        self.extra_sources = extra_sources
        self.have_constant_matrices = (len(pot_func) == 0)

    def pot_code(self):
        d = {}
        if self.pot_attr:
            d['have_master_slider'] = True
            d['master_slider_id'] = self.pot_attr[0][0]
        else:
            d['have_master_slider'] = False
        d['knob_ids'] = [t[0] for t in self.pot_attr]
        d['timecst'] = 0.01
        d['regs'] = [dict(id=vv[0],name=vv[1],desc="",varidx=i) for i, vv in enumerate(self.pot_attr)]
        ll = []
        for i, (var, name, loga, inv, expr) in enumerate(self.pot_attr):
            if loga and inv:
                ss = "t[%d] = (exp(%s * (1-self.pots[%d])) - 1) / (exp(%s) - 1);" % (i, loga, i, loga)
            elif loga:
                ss = "t[%d] = (exp(%s * self.pots[%d]) - 1) / (exp(%s) - 1);" % (i, loga, i, loga)
            elif inv:
                ss = "t[%d] = 1-self.pots[%d];" % (i, i)
            else:
                ss = "t[%d] = self.pots[%d];" % (i, i)
            ll.append(ss)
        s = 0.993;
        d['calc_pots'] = "\n    ".join(ll)
        return d

    @staticmethod
    def walk_eqs(nlin):
        yield nlin
        if nlin.__class__ is dk_simulator.NonlinEquations:
            return
        for bl in nlin.subblocks:
            for neq in CodeGenerator.walk_eqs(bl):
                yield neq

    def gen_nonlin(self, nonlin, s, d, complist, code):
        eq = self.eq
        if isinstance(nonlin, dk_simulator.PartitionedNonlinEquations):
            for i, nlin in enumerate(nonlin.subblocks):
                #spar = solver_set_defaults({} if self.solver_params is None else self.solver_params[i])
                self.gen_nonlin(nlin, s, d, complist, code)
            if self.solver_dict["method"] == "table":
                solver = self.solver_dict.copy()
                solver["method"] = "hybrCC"
            else:
                solver = self.solver_dict
            code.append(NonlinCode(s, nonlin, solver, self.extra_sources).generate(d))
        elif isinstance(nonlin, dk_simulator.ChainedNonlinEquations):
            for i, nlin in enumerate(nonlin.subblocks):
                #spar = solver_set_defaults({} if self.solver_params is None else self.solver_params[i])
                self.gen_nonlin(nlin, s, d, complist, code)
            solver = self.solver_dict
            code.append(NonlinChained(s, nonlin).generate(d))
        else:
            complist.append(dict(
                name = nonlin.name,
                namespace = nonlin.namespace,
                pins = ",".join(dk_simulator.Parser.format_element(v) for v in nonlin.pins),
                v_slice = nonlin.v_slice,
                p_slice = nonlin.p_slice,
                i_slice = nonlin.i_slice,
                ))
            generator = None
            d.overwrite("v0_guess", "")
            if self.solver_params:
                try:
                    sp = self.solver_params[nonlin.name]
                except KeyError:
                    pass
                except TypeError:
                    pass
                else:
                    generator = sp.get("generator")
                    d.overwrite("v0_guess", sp.get("v0_guess"))
            if generator:
                code.append(generator(s, nonlin, self.extra_sources, d))
            elif self.solver_dict["method"] == "table":
                code.append(TableCode(s, nonlin, self.extra_sources).generate(d))
            else:
                code.append(NonlinCode(s, nonlin, self.solver_dict, None).generate(d))

    def add_dict(self, d):
        eq = self.eq
        s = Structure("nonlin_param")
        par_ini = {}
        glob = Namespace()
        if self.have_constant_matrices:
            d["update_pot"] = ""
        else:
            d.update(UpdateMatrix(glob, eq, self.pot, self.pot_list, self.pot_func, self.Pv).generate(s))
        if eq.nonlin and eq.nonlin.nno:
            for k, v in self.solver_dict.items():
                d["solver_"+k] = v
            s.add(MatrixDeclaration('p', rows=eq.nn, cols=1, pointer=True))
            s.add(MatrixDeclaration('i', rows=eq.nn, cols=1, pointer=True))
            s.add(MatrixDeclaration('v', rows=eq.nn, cols=1, mapping=True, pointer=True, aligned=False))
            s.add_nonlin_info()
            s.add_minmax_info(eq.nonlin.nni)
            par_ini.update(dict(
                v = '&g_v',
                info = '&g_info',
                fnorm = '0',
                nfev = '&g_nfev',
                p = '0',
                i = '0',
                p_val = '0',
                ))
            ini = dict(
                p = MatrixAccess('mp'),
                i = MatrixAccess('mi'),
                v = '&g_v',
                info = '&g_info',
                fnorm = '&fnorm',
                nfev = '&g_nfev',
                p_val = MatrixAccess('p_val'),
                )
            d["nonlin_mat_list_calc"] = s.get_initializer(ini)
            for i, neq in enumerate(self.walk_eqs(eq.nonlin)):
                if i == 0:
                    neq.namespace = "nonlin"
                else:
                    neq.namespace = "nonlin_%d" % (i-1)
            complist = []
            code = []
            self.gen_nonlin(eq.nonlin, s, d, complist, code)
            d["nc"] = len(complist)
            d["components"] = complist
            d["nonlin_code"] = "".join(code)
            if eq.nn != eq.nonlin.nno:
                d["iblock"] = VectorAccess.block_expr([0, eq.nonlin.nno])
            else:
                d["iblock"] = ""
        else:
            d["nonlin_code"] = ""
            d["nc"] = 0
            d["iblock"] = ""
        d["nonlin_mat_list"] = s.get_initializer(par_ini)
        d.update(self.pot_code())
        d["struct_def"] = s
        d.update(LinearCode(glob, eq).generate())
        d["global_matrices"] = glob
        d["add_npl"] = 0
        d["DKPlugin_fields"] = ""
        d["DKPlugin_init"] = ""
        d["process_add"] = ""
        return d

    def generate(self, d):
        d = self.add_dict(d)
        d["lv2_ports"] = LV2_Port_List(self.pot_attr, self.pot, self.eq)
        out = dict(c_source = dk_templates.c_template_top.render(d))
        plugindef = d["plugindef"]
        if plugindef.lv2_plugin_type:
            out["manifest.ttl"] = dk_templates.lv2_manifest.render(d)
            out["%s.ttl" % plugindef.lv2_versioned_id] = dk_templates.lv2_ttl.render(d)
        return out



#! /usr/bin/env python
# encoding: utf-8

import os,shutil,sys,platform
import TaskGen,Task,Build,Options,Utils
from TaskGen import taskgen,feature,after,before
from Logs import error,debug
app_info='''
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE plist SYSTEM "file://localhost/System/Library/DTDs/PropertyList.dtd">
<plist version="0.9">
<dict>
	<key>CFBundlePackageType</key>
	<string>APPL</string>
	<key>CFBundleGetInfoString</key>
	<string>Created by Waf</string>
	<key>CFBundleSignature</key>
	<string>????</string>
	<key>NOTE</key>
	<string>THIS IS A GENERATED FILE, DO NOT MODIFY</string>
	<key>CFBundleExecutable</key>
	<string>%s</string>
</dict>
</plist>
'''
def set_macosx_deployment_target(self):
	if self.env['MACOSX_DEPLOYMENT_TARGET']:
		os.environ['MACOSX_DEPLOYMENT_TARGET']=self.env['MACOSX_DEPLOYMENT_TARGET']
	elif'MACOSX_DEPLOYMENT_TARGET'not in os.environ:
		if sys.platform=='darwin':
			os.environ['MACOSX_DEPLOYMENT_TARGET']='.'.join(platform.mac_ver()[0].split('.')[:2])
def apply_framework(self):
	for x in self.to_list(self.env['FRAMEWORKPATH']):
		frameworkpath_st='-F%s'
		self.env.append_unique('CXXFLAGS',frameworkpath_st%x)
		self.env.append_unique('CCFLAGS',frameworkpath_st%x)
		self.env.append_unique('LINKFLAGS',frameworkpath_st%x)
	for x in self.to_list(self.env['FRAMEWORK']):
		self.env.append_value('LINKFLAGS',['-framework',x])
def create_bundle_dirs(self,name,out):
	bld=self.bld
	dir=out.parent.get_dir(name)
	if not dir:
		dir=out.__class__(name,out.parent,1)
		bld.rescan(dir)
		contents=out.__class__('Contents',dir,1)
		bld.rescan(contents)
		macos=out.__class__('MacOS',contents,1)
		bld.rescan(macos)
	return dir
def bundle_name_for_output(out):
	name=out.name
	k=name.rfind('.')
	if k>=0:
		name=name[:k]+'.app'
	else:
		name=name+'.app'
	return name
def create_task_macapp(self):
	if self.env['MACAPP']or getattr(self,'mac_app',False):
		apptask=self.create_task('macapp')
		apptask.set_inputs(self.link_task.outputs)
		out=self.link_task.outputs[0]
		name=bundle_name_for_output(out)
		dir=self.create_bundle_dirs(name,out)
		n1=dir.find_or_declare(['Contents','MacOS',out.name])
		apptask.set_outputs([n1])
		apptask.chmod=0755
		apptask.install_path=os.path.join(self.install_path,name,'Contents','MacOS')
		self.apptask=apptask
def create_task_macplist(self):
	if self.env['MACAPP']or getattr(self,'mac_app',False):
		if not getattr(self,'mac_plist',False):
			self.mac_plist=app_info
		plisttask=self.create_task('macplist')
		plisttask.set_inputs(self.link_task.outputs)
		out=self.link_task.outputs[0]
		self.mac_plist=self.mac_plist%(out.name)
		name=bundle_name_for_output(out)
		dir=self.create_bundle_dirs(name,out)
		n1=dir.find_or_declare(['Contents','Info.plist'])
		plisttask.set_outputs([n1])
		plisttask.mac_plist=self.mac_plist
		plisttask.install_path=os.path.join(self.install_path,name,'Contents')
		self.plisttask=plisttask
def apply_link_osx(self):
	name=self.link_task.outputs[0].name
	if not self.install_path:
		return
	if getattr(self,'vnum',None):
		name=name.replace('.dylib','.%s.dylib'%self.vnum)
	path=os.path.join(Utils.subst_vars(self.install_path,self.env),name)
	if'-dynamiclib'in self.env['LINKFLAGS']:
		self.env.append_value('LINKFLAGS','-install_name')
		self.env.append_value('LINKFLAGS',path)
def apply_bundle(self):
	if not('cshlib'in self.features or'shlib'in self.features):return
	if self.env['MACBUNDLE']or getattr(self,'mac_bundle',False):
		self.env['shlib_PATTERN']=self.env['macbundle_PATTERN']
		uselib=self.uselib=self.to_list(self.uselib)
		if not'MACBUNDLE'in uselib:uselib.append('MACBUNDLE')
def apply_bundle_remove_dynamiclib(self):
	if self.env['MACBUNDLE']or getattr(self,'mac_bundle',False):
		if not getattr(self,'vnum',None):
			try:
				self.env['LINKFLAGS'].remove('-dynamiclib')
			except ValueError:
				pass
app_dirs=['Contents','Contents/MacOS','Contents/Resources']
def app_build(task):
	env=task.env
	shutil.copy2(task.inputs[0].srcpath(env),task.outputs[0].abspath(env))
	return 0
def plist_build(task):
	env=task.env
	f=open(task.outputs[0].abspath(env),"w")
	f.write(task.mac_plist)
	f.close()
	return 0
Task.task_type_from_func('macapp',vars=[],func=app_build,after="cxx_link cc_link static_link")
Task.task_type_from_func('macplist',vars=[],func=plist_build,after="cxx_link cc_link static_link")

feature('cc','cxx')(set_macosx_deployment_target)
before('apply_lib_vars')(set_macosx_deployment_target)
feature('cc','cxx')(apply_framework)
after('apply_lib_vars')(apply_framework)
taskgen(create_bundle_dirs)
taskgen(create_task_macapp)
after('apply_link')(create_task_macapp)
feature('cprogram')(create_task_macapp)
after('apply_link')(create_task_macplist)
feature('cprogram')(create_task_macplist)
after('apply_link')(apply_link_osx)
feature('cshlib')(apply_link_osx)
before('apply_link','apply_lib_vars')(apply_bundle)
feature('cc','cxx')(apply_bundle)
after('apply_link')(apply_bundle_remove_dynamiclib)
feature('cshlib')(apply_bundle_remove_dynamiclib)



# -*- coding: utf-8 -*-
from pylab import *
from scipy.optimize import brentq
import sys

class PlotManager(object):
    def __init__(self, r, c):
        self.r = r
        self.c = c
        self.i = 1

    def __call__(self):
        subplot(self.r,self.c,self.i)
        self.i += 1

def f(D, x, d):
    Is = 10 * 1e-12
    mUt = 30 * 1e-3
    R2 = 51000 + 500000 * d
    return (x+D)/R2 + Is * (exp(D/mUt) - exp(-D/mUt))

def F(x, d):
    ret = brentq(f, -0.6, 0.6, args=(x, d))
    return ret

def xf(v):
    return v / (3.0 + v)

def fx(v):
    return 3.0 * v / (1.0 - v)
    
def show_diff(pm):
    from ts9func import ts9func
    d = 1.0
    pm()
    x = linspace(0, 100, 2000)
    y = vectorize(lambda v: F(v, d))(x)
    y1 = vectorize(lambda v: ts9func(v))(x)
    plot(x, y-y1, "-")
    pm()
    x = linspace(0, 2.5, 200)
    y = vectorize(lambda v: F(v, d))(x)
    y1 = vectorize(ts9func)(x)
    plot(x, y-y1, "-")

def write_ftube_table(Vi, Vp):
    """write C source"""
    sys.stdout.write('\t%g,%g,%g,%d, {' % (Vi[0], Vi[-1], (len(Vi)-1)/(Vi[-1]-Vi[0]), len(Vi)))
    s = ""
    for i, v in enumerate(Vi):
        if i % 5 == 0:
            sys.stdout.write(s+"\n\t")
            s = ""
        sys.stdout.write(s+str(Vp[i]))
        s = ","
    sys.stdout.write("\n\t}")

def make_table():
    Vi = linspace(xf(0.0),xf(100.0),100)
    sys.stdout.write("// file generated by tools/ts9sim.py -- do not modify manually\n")
    sys.stdout.write("\n")
    sys.stdout.write("static table1d_imp<%d> ts9table __rt_data = {\n" % len(Vi))
    Vp = vectorize(lambda v: F(fx(v), 1.0))(Vi)
    write_ftube_table(Vi, Vp)
    sys.stdout.write("\n};\n")

if len(sys.argv) == 2:
    make_table()
else:
    pm = PlotManager(2,1)
    show_diff(pm)
    show()



import os, sys; sys.path.append("../build/default/pygxw")
import gxw, gtk

rc_style = """
pixmap_path "%s"

style "Wheel" {
  stock["wheel_back"] = {{"wheel_back.png"}}
  stock["wheel_fringe"] = {{"wheel_fringe.png"}}
  stock["wheel_pointer"] = {{"wheel_pointer.png"}}

}

style "ToggleImage" {
  stock["led_on"] = {{"led_on.png"}}
  stock["led_off"] = {{"led_off.png"}}
  stock["button_on"] = {{"button_on.png"}}
  stock["button_off"] = {{"button_off.png"}}
  stock["minitoggle_on"] = {{"minitoggle_on.png"}}
  stock["minitoggle_off"] = {{"minitoggle_off.png"}}
  stock["switchit_on"] = {{"switchit_on.png"}}
  stock["switchit_off"] = {{"switchit_off.png"}}
  stock["switch_on"] = {{"switch_on.png"}}
  stock["switch_off"] = {{"switch_off.png"}}
  GxRadioButton::indicator-size = 10
}

style "Switch" {
  xthickness = 0
  ythickness = 0
  GtkButton::inner-border = {0, 0, 0, 0}
  GtkButton::default-border = {0, 0, 0, 0}
  GtkButton::focus-line-width = 0
  GtkButton::focus-padding = 0
  GtkButton::interior-focus = 0
  bg[PRELIGHT] = { 0.03, 0.03, 0.03 }
}

style "BigKnob" {
  stock["knob"] = {{"knob1.png"}}
  GxKnob::arc-inset = 2
}

style "SmallKnob" {
  stock["smallknob"] = {{"smallknob1.png"}}
  GxKnob::arc-inset = 0
}

style "Slider" {
  stock["hslider"] = {{"slider.png"}}
  stock["minislider"] = {{"minislider.png"}}
  GtkRange::slider-width = 20
}

style "MiniSlider" {
  stock["minislider"] = {{"minislider.png"}}
  GtkRange::slider-width = 6
}

style "VSlider" {
  stock["vslider"] = {{"vslider.png"}}
  GtkRange::slider-width = 20
}

style "MiniSlider" {
  stock["minislider"] = {{"minislider.png"}}
  GtkRange::slider-width = 6
}

style "EqSlider" {
  stock["eqslider"] = {{"eqslider.png"}}
  GtkRange::slider-width = 5
}

class "GxWheel" style "Wheel"
class "GxToggleImage" style "ToggleImage"
class "GxSwitch" style "Switch"
class "GxBigKnob" style "BigKnob"
class "GxSmallKnob" style "SmallKnob"
class "GxHSlider" style "Slider"
class "GxMiniSlider" style "MiniSlider"
class "GxVSlider" style "VSlider"
class "GxEQSlider" style "EqSlider"
class "GxRadioButton" style "ToggleImage"
"""

rc_style = """
pixmap_path "%s"

style "BigKnob" {
  stock["bigknob"] = {{"knob3.png"}}
}

style "SmallKnob" {
  stock["smallknob"] = {{"smallknob5.png"}}
}

style "Switch" {
  xthickness = 0
  ythickness = 0
  GtkButton::focus-padding = 0
  bg_pixmap[NORMAL] = "<parent>"
}

style "ToggleImage" {
  bg_pixmap[NORMAL] = "<parent>"
}

style "PaintBox" {
    GxPaintBox::skin-gradient = {
        {    0, 65536, 65536, 65536, 58982},
        {52429, 58982, 58982, 65536, 58982},
        {65536, 52429, 52429, 58982, 32768}}
    GxPaintBox::paint-func = "rectangle_expose"
}


style "BigKnob" {
  stock["knob"] = {{"knob4.png"}}
  GxKnob::arc-inset = 2
}

class "GxBigKnob" style "BigKnob"
class "GxSmallKnob" style "SmallKnob"
class "GxSwitch" style "Switch"
class "GxPaintBox" style "PaintBox"
"""

def lm(db):
    if db < -70.0:
        df = 0.0
    elif db < -60.0:
        df = (db + 70.0) * 0.25
    elif db < -50.0:
        df = (db + 60.0) * 0.5 + 2.5
    elif db < -40.0:
        df = (db + 50.0) * 0.75 + 7.5
    elif db < -30.0:
        df = (db + 40.0) * 1.5 + 15.0
    elif db < -20.0:
        df = (db + 30.0) * 2.0 + 30.0
    elif db < 6.0:
        df = (db + 20.0) * 2.5 + 50.0
    else:
        df = 115.0
    return df/115.0

def demo(w):
    v = gtk.VBox()
    #h = MyBox()
    h = gxw.PaintBox()
    h.props.paint_func = "amp_expose"
    #h = gtk.HBox()
    h.props.border_width = 10
    h.props.spacing = 10
    w.add(v)

    r = gxw.Wheel(gtk.Adjustment(0,-999,999,0.1,10))
    r.props.show_value = True
    r.props.value_position = gtk.POS_LEFT
    h.pack_start(r,0,0)

    if True:
        for base in "led", "switch", "switchit", "minitoggle", "button":
            b = gxw.Switch(base)
            #b = gxw.ToggleImage()
            #print b.get_has_window()
            #b.props.base_name = base+"_off"
            h.pack_start(b,0,0)

    v.add(h)

    h = gtk.HBox()
    h.set_spacing(20)
    v.add(h)

    adj = gtk.Adjustment(0,-1,1,0.01,0.1)

    r = gxw.BigKnob(adj)
    r.props.show_value = True
    #r.props.value_position = gtk.POS_LEFT
    h.add(r)

    r = gxw.SmallKnob(adj)
    r.props.show_value = True
    r.props.value_position = gtk.POS_RIGHT
    h.add(r)

    r = gxw.HSlider(adj)
    r.props.show_value = True
    r.props.value_position = gtk.POS_LEFT
    h.add(r)

    r = gxw.MiniSlider(adj)
    h.add(r)

    r = gxw.EQSlider(adj)
    h.add(r)

    r = gxw.EQSlider(adj)
    r.props.show_value = True
    h.add(r)

    r = gxw.VSlider(adj)
    h.add(r)

    r = gxw.VSlider(adj)
    r.props.show_value = True
    h.add(r)

    r = gxw.Selector()
    model = gtk.ListStore(str)
    for s in "Amp 1", "Amp 2", "Amp 3":
        model.append((s,))
    r.props.model = model
    h.add(r)

    h1 = gtk.HBox()
    h.add(h1)

    r = gxw.FastMeter()
    r.set(0.6)
    h1.add(r)

    global ms
    r = ms = gxw.MeterScale()
    r.props.tick_pos = gxw.TICK_BOTH
    for p in -50, -40, -20, -30, -10, -3, 0, 4:
        r.add_mark(lm(p), str(p));
    h1.add(r)

    r = gxw.FastMeter()
    r.set(0.7)
    h1.add(r)

    h = gtk.HBox()
    h.set_spacing(20)
    r = None
    for i in range(3):
        r = gxw.RadioButton(r)
        #r = gtk.RadioButton(r)
        #r.props.image = gxw.ToggleImage("minitoggle")
        #r.props.draw_indicator = 0
        r.props.base_name = "minitoggle"
        r.set_label("bla %d" % i)
        h.pack_start(r,0,0)
    #print list(r.props)
    v.add(h)


#gtk.rc_parse("../rcstyles/guitarix_default.rc")
gtk.rc_parse_string(rc_style % os.path.abspath("../libgxw"))
w = gtk.Window()
#b = gxw.ToggleImage()
#w.add(b)
demo(w)
w.show_all()

try:
    __IPYTHON__
except:
    w.connect("destroy",gtk.main_quit)
    gtk.main()



from cython.operator cimport dereference as deref, preincrement as inc
import numpy as np
cimport numpy as np

cdef extern from "dlfcn.h":
    cdef enum:
        RTLD_LOCAL
        RTLD_GLOBAL
        RTLD_NOW
    void *dlopen(char *__file, int __mode)
    void *dlsym (void *__handle, char *__name)
    char *dlerror()
    void dlclose(void *)

cdef extern from "time.h":
    cdef struct timespec:
        int tv_sec
        int tv_nsec
    ctypedef int clockid_t
    int CLOCK_MONOTONIC
    int clock_gettime(clockid_t clk_id, timespec *tp)

DEF UseDouble = False

IF UseDouble:
    ctypedef double realt
    np_realt = np.float64
ELSE:
    ctypedef float realt
    np_realt = np.float32

ctypedef realt *realtp

cdef extern from "gx_plugin.h":
    cdef struct value_pair:
        char *value_id
        char *value_label
    ctypedef value_pair* const_value_pair_ptr "const value_pair*"
    cppclass Var:
        char *id
        char *name
        realt *var
        int   *ivar
        realt val
        realt low
        realt up
        value_pair *values
    cppclass string:
        char *c_str()
    cppclass pair:
        string first
        Var *second
    cppclass variter:
        variter operator++()
        bint operator!=(variter)
        bint operator==(variter)
        pair operator*()
    cppclass PluginDef
    cppclass VarMap:
        variter begin()
        variter end()
        variter find(char *pname)
    cppclass ParamRegImpl:
        PluginDef *plugin
        ParamRegImpl(VarMap *v, PluginDef *p)
    ctypedef int (*registerfunc)(ParamRegImpl& reg)
    ctypedef void (*process_mono_audio) (int count, realt *input, realt *output, PluginDef *plugin)
    ctypedef void (*inifunc)(unsigned int samplingFreq, PluginDef *plugin)
    ctypedef int (*activatefunc)(int start, PluginDef *plugin)
    ctypedef void (*clearstatefunc)(PluginDef *plugin)
    ctypedef void (*deletefunc)(PluginDef *plugin)
    ctypedef void (*process_stereo_audio)(int count, realt *input1, realt *input2,
                                           realt *output1, realt *output2, PluginDef *plugin)
    cppclass PluginDef:
        int version
        int flags
        char* id
        char* name
        char** groups
        char* decription
        char* category
        char* shortname
        registerfunc register_params
        process_mono_audio mono_audio
        process_stereo_audio stereo_audio
        inifunc set_samplerate
        activatefunc activate_plugin
        clearstatefunc clear_state
        deletefunc delete_instance
    ctypedef int (*plugin_inifunc)(unsigned int idx, PluginDef **p)

cdef extern from "pluginloader.h":
    int compare_plugin_version(PluginDef *p)

cdef inline double ts_diff(timespec ts1, timespec ts2):
    cdef double df = ts1.tv_sec - ts2.tv_sec
    return df * 1e9 + (ts1.tv_nsec - ts2.tv_nsec)

cdef class Plugin:
    cdef void *handle
    cdef PluginDef *p
    cdef VarMap *varmap
    cdef dict d
    cdef double time_per_sample
    cdef int _samplerate

    def __cinit__(self, path, unsigned int idx = 0):
        cdef int n
        self.varmap = new VarMap()
        self.p = <PluginDef*>0
        if "/" not in path:
            path = "./" + path # do not search system dirs
        self.handle = dlopen(path, RTLD_LOCAL|RTLD_NOW)
        if not self.handle:
            raise RuntimeError("Cannot open library: %s [%s]" % (path, dlerror()))
        cdef plugin_inifunc get_gx_plugin = <plugin_inifunc>dlsym(self.handle, "get_gx_plugin")
        cdef char *dlsym_error = dlerror()
        if dlsym_error:
            dlclose(self.handle)
            self.handle = <void*>0
            raise RuntimeError("Cannot load symbol 'get_gx_plugin': %s" % dlsym_error)
        n = get_gx_plugin(0, <PluginDef**>0)
        if n < 0:
            raise RuntimeError("error loading plugins")
        if n == 0:
            raise RuntimeError("no plugin found")
        if idx >= <unsigned int>n:
            raise RuntimeError("idx too big (%d > %d)" % (idx, n-1))
        if get_gx_plugin(idx, &self.p) < 0:
            raise RuntimeError("plugin loader error")
        if not compare_plugin_version(self.p):
            raise ValueError("bad plugin version 0x%x" % self.p[0].version)
        cdef ParamRegImpl *pr
        cdef variter i
        d = {}
        if self.p[0].register_params:
            pr = new ParamRegImpl(self.varmap, &self.p[0])
            self.p[0].register_params(pr[0])
            del pr
            i = self.varmap[0].begin()
            while i != self.varmap[0].end():
                d[deref(i).second[0].id] = (
                    deref(i).second[0].name, deref(i).second[0].val,
                    deref(i).second[0].low, deref(i).second[0].up,
                    self.get_vp_list(deref(i).second))
                inc(i)
        self.d = d
        self.time_per_sample = 0
        self._samplerate = 0

    property nanosec_per_sample:
        "time in nanoseconds measured for last compute call"
        def __get__(self):
            return self.time_per_sample

    def keys(self):
        "List of parameters names"
        return self.d.keys()

    def get_var_attr(self, key):
        """argument: parameter name
        returns: name, stdval, lower, upper"""
        return self.d[key]

    cdef get_vp_list(self, Var *p):
        # typecast needed because assignment uses variable without const internally (bug?)
        cdef const_value_pair_ptr vp = <value_pair*>p.values
        if not vp:
            return []
        l = []
        for i in range(int(p.up)+1):
            l.append((vp[i].value_id, vp[i].value_label))
        return l

    def get_range(self, key):
        "return (lower,upper) range for parameter name"
        f = self.d[key]
        return (f[2], f[3])

    def __getitem__(self, char *pname):
        cdef variter p = self.varmap[0].find(pname)
        if p == self.varmap[0].end():
            raise KeyError("not found: %s" % pname)
        if deref(p).second.var:
            return deref(p).second.var[0]
        if deref(p).second.ivar:
            return deref(p).second.ivar[0]
        assert(False)

    def __setitem__(self, char* pname, pval):
        cdef variter p = self.varmap[0].find(pname)
        if p == self.varmap[0].end():
            raise KeyError("not found: %s" % pname)
        if not (deref(p).second[0].low <= pval <= deref(p).second[0].up):
            raise ValueError(
                "parameter %s value %s outside range [%s, %s]"
                % (pname, pval, deref(p).second[0].low, deref(p).second[0].up))
        if deref(p).second[0].var:
            deref(p).second[0].var[0] = pval
        elif deref(p).second[0].ivar:
            deref(p).second[0].ivar[0] = pval
        else:
            assert(False)

    property num_inputs:
        "number of input channels"
        def __get__(self):
            if self.p[0].mono_audio:
                return 1
            elif self.p[0].stereo_audio:
                return 2
            else:
                return 0

    property num_outputs:
        "number of output channels"
        def __get__(self):
            if self.p[0].mono_audio:
                return 1
            elif self.p[0].stereo_audio:
                return 2
            else:
                return 0

    property samplerate:
        "initialized samplerate"
        def __get__(self):
            return self._samplerate

    def init(self, int samplingRate):
        """argument: sample rate
        must be called before calling compute"""
        if self.p[0].activate_plugin:
            self.p[0].activate_plugin(True, self.p)
        if self.p[0].set_samplerate:
            self.p[0].set_samplerate(samplingRate, self.p)
        self._samplerate = samplingRate

    def clear_state(self):
        if self.p[0].clear_state:
            self.p[0].clear_state(self.p)

    def compute(self, np.ndarray inp_data not None):
        """execute dsp algorithm

        argument: numpy array
        returns: float32 numpy array

        specify type when creating an array, e.g. zeros(200, dtype=float32)
        """
        if not self._samplerate:
            raise ValueError("Plugin not initialized")
        cdef int transposed = False
        if self.p[0].mono_audio:
            if np.PyArray_NDIM(inp_data) == 2:
                if np.PyArray_DIMS(inp_data)[0] != 1:
                    if np.PyArray_DIMS(inp_data)[1] == 1:
                        transposed = True
                        inp_data = inp_data.T
                    else:
                        raise ValueError("plugin has only channel")
            elif not np.PyArray_NDIM(inp_data) == 1:
                raise ValueError("plugin has only channel")
        elif self.p[0].stereo_audio:
            if np.PyArray_NDIM(inp_data) != 2:
                raise ValueError("plugin has 2 channels")
            if np.PyArray_DIMS(inp_data)[0] != 2 and np.PyArray_DIMS(inp_data)[1] == 2:
                transposed = True
                inp_data = inp_data.T
        else:
            raise ValueError("no process function available")
        cdef np.ndarray inp = np.array(inp_data, dtype=np_realt, order='C', copy=False)
        cdef int count = np.PyArray_DIMS(inp)[np.PyArray_NDIM(inp)-1]
        cdef timespec t0, t1
        cdef np.ndarray o
        if self.p[0].mono_audio:
            o = np.empty(count,dtype=np_realt)
            clock_gettime(CLOCK_MONOTONIC, &t0)
            self.p[0].mono_audio(count, <realtp>np.PyArray_DATA(inp), <realtp>np.PyArray_DATA(o), self.p)
            clock_gettime(CLOCK_MONOTONIC, &t1)
            if np.PyArray_NDIM(inp_data) == 2:
                o = o.reshape(1, count)
        else:
            o = np.empty((2,count),dtype=np_realt)
            clock_gettime(CLOCK_MONOTONIC, &t0)
            self.p[0].stereo_audio(
                count, <realtp>np.PyArray_DATA(inp), <realtp>(<char*>np.PyArray_DATA(inp)+np.PyArray_STRIDES(inp)[0]),
                <realtp>np.PyArray_DATA(o), <realtp>(<char*>np.PyArray_DATA(o)+np.PyArray_STRIDES(o)[0]), self.p)
            clock_gettime(CLOCK_MONOTONIC, &t1)
        self.time_per_sample = ts_diff(t1,t0)/count
        if transposed:
            o = o.T
        return o

    def __dealloc__(self):
        del self.varmap
        if self.handle:
            if self.p and self.p[0].delete_instance:
                self.p[0].delete_instance(self.p)
            dlclose(self.handle)



#! /usr/bin/env python
# encoding: utf-8

import sys
import Utils,ar
from Configure import conftest
def find_dmd(conf):
	conf.find_program(['dmd','ldc'],var='D_COMPILER',mandatory=True)
def common_flags_ldc(conf):
	v=conf.env
	v['DFLAGS']=['-d-version=Posix']
	v['DLINKFLAGS']=[]
	v['D_shlib_DFLAGS']=['-relocation-model=pic']
def common_flags_dmd(conf):
	v=conf.env
	v['DFLAGS']=['-version=Posix']
	v['D_SRC_F']=''
	v['D_TGT_F']=['-c','-of']
	v['DPATH_ST']='-I%s'
	v['D_LINKER']=v['D_COMPILER']
	v['DLNK_SRC_F']=''
	v['DLNK_TGT_F']='-of'
	v['DLIB_ST']='-L-l%s'
	v['DLIBPATH_ST']='-L-L%s'
	v['DFLAGS_OPTIMIZED']=['-O']
	v['DFLAGS_DEBUG']=['-g','-debug']
	v['DFLAGS_ULTRADEBUG']=['-g','-debug']
	v['DLINKFLAGS']=['-quiet']
	v['D_shlib_DFLAGS']=['-fPIC']
	v['D_shlib_LINKFLAGS']=['-L-shared']
	v['DHEADER_ext']='.di'
	v['D_HDR_F']=['-H','-Hf']
def detect(conf):
	conf.find_dmd()
	conf.check_tool('ar')
	conf.check_tool('d')
	conf.common_flags_dmd()
	conf.d_platform_flags()
	if conf.env.D_COMPILER.find('ldc')>-1:
		conf.common_flags_ldc()

conftest(find_dmd)
conftest(common_flags_ldc)
conftest(common_flags_dmd)



#! /usr/bin/env python
# encoding: utf-8

import os
import Task,Options,Utils
from Configure import conf
from TaskGen import extension,taskgen,feature,before
xsubpp_str='${PERL} ${XSUBPP} -noprototypes -typemap ${EXTUTILS_TYPEMAP} ${SRC} > ${TGT}'
EXT_XS=['.xs']
def init_perlext(self):
	self.uselib=self.to_list(getattr(self,'uselib',''))
	if not'PERL'in self.uselib:self.uselib.append('PERL')
	if not'PERLEXT'in self.uselib:self.uselib.append('PERLEXT')
	self.env['shlib_PATTERN']=self.env['perlext_PATTERN']
def xsubpp_file(self,node):
	outnode=node.change_ext('.c')
	self.create_task('xsubpp',node,outnode)
	self.allnodes.append(outnode)
Task.simple_task_type('xsubpp',xsubpp_str,color='BLUE',before='cc cxx',shell=False)
def check_perl_version(conf,minver=None):
	if getattr(Options.options,'perlbinary',None):
		conf.env.PERL=Options.options.perlbinary
	else:
		conf.find_program('perl',var='PERL',mandatory=True)
	try:
		version=Utils.cmd_output([conf.env.PERL,'-e','printf "%vd",$^V'])
	except:
		conf.fatal('could not determine the perl version')
	conf.env.PERL_VERSION=version
	cver=''
	if minver:
		try:
			ver=tuple(map(int,version.split('.')))
		except:
			conf.fatal('unsupported perl version %r'%version)
		if ver<minver:
			conf.fatal('perl is too old')
		cver='.'.join(map(str,minver))
	conf.check_message('perl',cver,True,version)
def check_perl_module(conf,module):
	cmd=[conf.env['PERL'],'-e','use %s'%module]
	r=Utils.pproc.call(cmd,stdout=Utils.pproc.PIPE,stderr=Utils.pproc.PIPE)==0
	conf.check_message("perl module %s"%module,"",r)
	return r
def check_perl_ext_devel(conf):
	if not conf.env.PERL:
		conf.fatal('perl detection is required first')
	def read_out(cmd):
		return Utils.to_list(Utils.cmd_output([conf.env.PERL,'-MConfig','-e',cmd]))
	conf.env.LINKFLAGS_PERLEXT=read_out('print $Config{lddlflags}')
	conf.env.CPPPATH_PERLEXT=read_out('print "$Config{archlib}/CORE"')
	conf.env.CCFLAGS_PERLEXT=read_out('print "$Config{ccflags} $Config{cccdlflags}"')
	conf.env.XSUBPP=read_out('print "$Config{privlib}/ExtUtils/xsubpp$Config{exe_ext}"')
	conf.env.EXTUTILS_TYPEMAP=read_out('print "$Config{privlib}/ExtUtils/typemap"')
	conf.env.perlext_PATTERN='%s.'+read_out('print $Config{dlext}')[0]
	if getattr(Options.options,'perlarchdir',None):
		conf.env.ARCHDIR_PERL=Options.options.perlarchdir
	else:
		conf.env.ARCHDIR_PERL=read_out('print $Config{sitearch}')[0]
def set_options(opt):
	opt.add_option("--with-perl-binary",type="string",dest="perlbinary",help='Specify alternate perl binary',default=None)
	opt.add_option("--with-perl-archdir",type="string",dest="perlarchdir",help='Specify directory where to install arch specific files',default=None)

before('apply_incpaths','apply_type_vars','apply_lib_vars')(init_perlext)
feature('perlext')(init_perlext)
extension(EXT_XS)(xsubpp_file)
conf(check_perl_version)
conf(check_perl_module)
conf(check_perl_ext_devel)



#! /usr/bin/env python
# encoding: utf-8

import Options
import Task
def set_options(opt):
	opt.add_option('--with-gcov',action='store_true',help='Use gcov for profiling',default=False)
def setup(bld):
	for name in('cc','cxx'):
		if name not in Task.TaskBase.classes:
			continue
		task=Task.TaskBase.classes[name]
		line=task.run.code.replace('SRC','SRC[0].abspath(env)')
		(fun,v)=Task.compile_fun(name,line,False)
		fun.code=line
		task.run=fun
def detect(conf):
	conf.find_program('gcov')
	gcov_flags='-fprofile-arcs -ftest-coverage'.split()
	conf.env['CXXFLAGS']=conf.env['CFLAGS']=conf.env['LINKFLAGS']=gcov_flags




#! /usr/bin/env python
# encoding: utf-8

import os,sys
import Configure,Options,Utils
import ccroot,ar
from Configure import conftest
def find_xlc(conf):
	cc=conf.find_program(['xlc_r','xlc'],var='CC',mandatory=True)
	cc=conf.cmd_to_list(cc)
	conf.env.CC_NAME='xlc'
	conf.env.CC=cc
def find_cpp(conf):
	v=conf.env
	cpp=None
	if v['CPP']:cpp=v['CPP']
	elif'CPP'in conf.environ:cpp=conf.environ['CPP']
	if not cpp:cpp=v['CC']
	v['CPP']=cpp
def xlc_common_flags(conf):
	v=conf.env
	v['CCFLAGS_DEBUG']=['-g']
	v['CCFLAGS_RELEASE']=['-O2']
	v['CC_SRC_F']=''
	v['CC_TGT_F']=['-c','-o','']
	v['CPPPATH_ST']='-I%s'
	if not v['LINK_CC']:v['LINK_CC']=v['CC']
	v['CCLNK_SRC_F']=''
	v['CCLNK_TGT_F']=['-o','']
	v['LIB_ST']='-l%s'
	v['LIBPATH_ST']='-L%s'
	v['STATICLIB_ST']='-l%s'
	v['STATICLIBPATH_ST']='-L%s'
	v['RPATH_ST']='-Wl,-rpath,%s'
	v['CCDEFINES_ST']='-D%s'
	v['SONAME_ST']=''
	v['SHLIB_MARKER']=''
	v['STATICLIB_MARKER']=''
	v['FULLSTATIC_MARKER']='-static'
	v['program_LINKFLAGS']=['-Wl,-brtl']
	v['program_PATTERN']='%s'
	v['shlib_CCFLAGS']=['-fPIC','-DPIC']
	v['shlib_LINKFLAGS']=['-G','-Wl,-brtl,-bexpfull']
	v['shlib_PATTERN']='lib%s.so'
	v['staticlib_LINKFLAGS']=''
	v['staticlib_PATTERN']='lib%s.a'
def detect(conf):
	conf.find_xlc()
	conf.find_cpp()
	conf.find_ar()
	conf.xlc_common_flags()
	conf.cc_load_tools()
	conf.cc_add_flags()

conftest(find_xlc)
conftest(find_cpp)
conftest(xlc_common_flags)



"""
support for generating .cc files from faust .dsp files with dsp2cc

for examples look at src/faust/wscript
"""

import Task, TaskGen, Logs, Utils, os, re, shutil

# task function for task "dsp"
def dsp2cc(task):
    # use the specified processor to create the output file and copy
    # the output file into faust-generated in the source tree
    src = task.inputs[0].srcpath(task.env)
    o = task.outputs[0]
    dst = o.bldpath(task.env)
    cpy = os.path.join(o.parent.srcpath(task.env)+task.gen_dir_suffix,o.file())
    lst = [task.proc, "-o", dst]
    if len(task.outputs) == 2:
        o_h = task.outputs[1]
        dst_h = o_h.bldpath(task.env)
        cpy_h = os.path.join(o_h.parent.srcpath(task.env)+task.gen_dir_suffix,o_h.file())
        lst += ["-H", dst_h]
    else:
        cpy_h = None
    lst += task.proc_args + [src]
    Logs.debug("runner: system command -> %s" % " ".join(lst))
    ret = Utils.exec_command(lst,shell=False)
    if ret != 0:
        return ret
    try:
        shutil.copy2(dst, cpy)
        if cpy_h:
            shutil.copy2(dst_h, cpy_h)
    except (OSError, IOError) as e:
        Logs.error("runner: cannot copy file -> %s" % e)
        return e.errno
    return 0

class FaustScanner(object):

    scan_re = "|".join([
        r'\bimport\s*\(\s*"(?P<I>[^"]+)"\s*\)\s*;',
        r'\bcomponent\s*\(\s*"(?P<C>[^"]+)"\s*\)',
        ])

    def scan_file(self, task, node):
        for s in "_ui.cc", "_ui.glade":
            n = node.parent.find_resource(node.name.replace(".dsp",s))
            if n:
                self.deps.append(n)
        pname = node.parent.abspath(task.env)
        for m in re.finditer(self.scan_re, node.read(task.env)):
            for fname in m.groupdict().values():
                if not fname:
                    continue
                key = (pname, fname)
                if key in self.scanned:
                    continue
                self.scanned.add(key)
                n = node.parent.find_resource(fname)
                if n:
                    self.deps.append(n)
                    self.scan_file(task, n)

    def scan(self, task):
        self.deps = []
        self.scanned = set()
        for node in task.inputs:
            self.scan_file(task, node)
        return self.deps, []

dsp_scanner = FaustScanner()

def scan_dsp(task):
    return dsp_scanner.scan(task)

# definition of task "dsp"
Task.task_type_from_func(
    name    = 'dsp',
    func    = dsp2cc,
    color   = 'BLUE',
    ext_in  = '.dsp',
    ext_out = '.cc',
    before  = 'cc cxx',
    ).scan = scan_dsp

@TaskGen.extension('.dsp')
def dsp_file(self, node):
    tsk = self.create_task('dsp')
    tsk.proc = self.proc
    tsk.proc_args = getattr(self, "proc_args", [])
    tsk.gen_dir_suffix = getattr(self, "gen_dir_suffix", "-generated")
    tsk.set_inputs(node)
    tsk.dep_vars = ['DSP2CC_ARGS']
    tsk.env['DSP2CC_ARGS'] = tsk.proc_args
    parent = node.parent
    o = node.change_ext('.cc')
    tsk.set_outputs(o)
    if getattr(self, "separate_header", False):
        o = node.change_ext('.h')
        tsk.set_outputs(o)
    self.bld.add_manual_dependency(node,self.bld.bldnode.find_resource(self.proc))



from __future__ import division
import os, quik


class MyLoader(object):

    def load_text(self, fname):
        if fname.startswith("%"):
            # hack for directly including guitarix gx_plugin.h
            guitarix_headers = "../../../src/headers"
            with open(os.path.join(guitarix_headers, fname[1:])) as f:
                return "".join([line for line in f if not (line.startswith("#include") or line.startswith("#pragma once"))])
        else:
            with open(fname) as f:
                return f.read()

    def load_template(self, name):
        return globals()[name]


class Template(quik.Template):
    def render(self, namespace, loader=MyLoader()):
        return quik.Template.render(self, namespace, loader)

c_template_top = Template("""\
%if (@build_script)\
// generated by @build_script
%else\
// generated automatically
%end
// DO NOT MODIFY!
#include <iostream>
#include <cmath>
#include <list>
%% %if (@method == "hybr" || @method == "lm")
%if (@method == "lm")
#include <cminpack.h>
%else
#include <float.h>
%end
#include <Eigen/Core>
%if (@np > 0)
#include <Eigen/LU>
%end
%if (@dev_interface)%% just for convenience, to make stand alone sources
%include ("%gx_compiler.h")
%include ("%gx_plugin.h")
%else
#include "gx_compiler.h"
#include "gx_plugin.h"
%end

using namespace Eigen;

#define N_(x) (x)

#define creal @c_real

static inline int sign(creal v) {
    return v < 0 ? -1 : 1;
}

static inline int Heaviside(creal v) {
    return v < 0 ? 0 : 1;  // Heaviside(0) == 1/2 in sympy but shouldn't matter
}

static Matrix<creal, @nx, 1> g_x;
creal g_v_data[@nn];
static Map<Matrix<creal, @nn, 1> >g_v(g_v_data);
static int g_info;
static int g_nfev;

%if (@dev_interface)
static creal g_fnorm;
static Array<creal, @nni, 1> g_min;
static Array<creal, @nni, 1> g_max;

#define INTERFACE_VERSION 5

extern "C" __attribute__ ((visibility ("default")))
int get_interface_version() {
    return INTERFACE_VERSION;
}

extern "C" __attribute__ ((visibility ("default")))
void get_structure(const char **name, int *data_size, int *samplerate, const int **shapes, const int (**comp_sz)[6],
                   const char ***pins, const char ***comp_names, const char **method, const char ***pot_vars,
                   const double **pot, const char ***out_labels, const char **comment) {
    static const char *n = "@name";
    static int sz[] = { @nx, @ni, @no, @npl, @nn, @nni, @nno, @nc, -1 }; // nx, ni, no, npl, nn, nni, nno, nc, -1
    static int nn_sz[][6] = { // component v_slice, p_slice, i_slice
    %for @c in @components:\
        {@c.v_slice.start, @c.v_slice.stop, @c.p_slice.start, @c.p_slice.stop, @c.i_slice.start, @c.i_slice.stop},
    %end
    };
    static const char *nn_name[] = { // component names
    %for @c in @components:\
        "@c.name", "@c.namespace",
    %end 0
    };
    static const char *nn_pins[] = { // component pins
    %for @c in @nlin_elements:\
        "@c",
    %end 0
    };
    static const char *m = "@method";
    static const char *pvars[] = {@pot_vars};
    static double pvalues[] = {@pot};
    static const char *ol[] = {@out_labels};
    static const char *c = "@comment";
    if (name) *name = n;
    if (data_size) *data_size = sizeof(creal);
    if (samplerate) *samplerate = @fs;
    if (shapes) *shapes = sz;
    if (comp_sz) *comp_sz = nn_sz;
    if (pins) *pins = nn_pins;
    if (comp_names) *comp_names = nn_name;
    if (method) *method = m;
    if (pot_vars) *pot_vars = pvars;
    if (pot) *pot = pvalues;
    if (out_labels) *out_labels = ol;
    if (comment) *comment = c;
}
%end

static creal x0_data[] = {@x0_data};
%if (@dev_interface)
static creal v0_data[] = {@v0_data};
static creal p0_data[] = {@p0_data};
static creal o0_data[] = {@o0_data};
static creal op_data[] = {@op_data};

extern "C" __attribute__ ((visibility ("default")))
void get_dc(creal *v0, creal *x0, creal *p0, creal *o0, creal *op) {
    if (v0) {
        for (int i = 0; i < @nn; i++) {
            v0[i] = v0_data[i];
        }
    }
    if (x0) {
        for (int i = 0; i < @nx; i++) {
            x0[i] = x0_data[i];
        }
    }
    if (p0) {
        for (int i = 0; i < @nni; i++) {
            p0[i] = p0_data[i];
        }
    }
    if (o0) {
        for (int i = 0; i < @no; i++) {
            o0[i] = o0_data[i];
        }
    }
    if (op) {
        for (int i = 0; i < @ni; i++) {
            op[i] = op_data[i];
        }
    }
}

extern "C" __attribute__ ((visibility ("default")))
void get_info(creal *v, creal *x, creal *minval, creal *maxval, int *info, int *nfev, creal *fnorm) {
    Map<Matrix<creal, @nn, 1> > V(v);
    V = g_v;
    Map<Matrix<creal, @nx, 1> > X(x);
    X = g_x;
    Map<Matrix<creal, @nni, 1> > Mi(minval);
    Mi = g_min;
    Map<Matrix<creal, @nni, 1> > Ma(maxval);
    Ma = g_max;
    *info = g_info;
    *nfev = g_nfev;
    *fnorm = g_fnorm;
}

extern "C" __attribute__ ((visibility ("default")))
void set_state(creal *v, creal *x) {
    Map<Matrix<creal, @nn, 1> > V(v);
    g_v = V;
    Map<Matrix<creal, @nx, 1> > X(x);
    g_x = X;
}
%end

#define real realtype  // real conflicts with Eigen::real of new eigen library version
typedef double real;
%if (@method == "table")
%include ("intpp.h")
#define NO_INTPP_INCLUDES
%include ("intpp.cc")
@extra_sources.intpp_inst
%%@extra_sources.data_h
@extra_sources.data_c
%end

@global_matrices

static Matrix<creal, @npl, 1> last_pot;

@struct_def

%if (@dev_interface || @npl)
void calc_inv_update(const creal *pot, nonlin_param& par) {
    Map<const Matrix<creal, @npl, 1> >pm(pot);
    last_pot = pm;
    @update_pot
}
%end
%if (@dev_interface)
nonlin_param par@nonlin_mat_list;

extern "C" __attribute__ ((visibility ("default")))
void calc_inv_update(const creal *pot) {
    calc_inv_update(pot, par);
}
%end

@nonlin_code

%if (@dev_interface)
extern "C" __attribute__ ((visibility ("default")))
int calc_stream(creal *u, creal *o, int n, int ii) {
    Matrix<creal, @nn, 1> mi;
%if (@nn)
    g_min = Matrix<creal, @nni, 1>::Constant(HUGE_VAL);
    g_max = Matrix<creal, @nni, 1>::Constant(-HUGE_VAL);
    g_fnorm = 0;
    par.v = &g_v;
    par.i = &mi;
    Array<creal, @nni, 1> p_val;
    par.p_val = &p_val;
    creal fnorm;
    par.fnorm = &fnorm;
    Matrix<creal, @nn, 1> mp;
    par.p = &mp;
%end
    int nu = @ni;
    if (ii >= 0) {
        nu += 1;
    }
    for (int j = 0; j < n; j++) {
#define GET_U (u+j*nu)
#define DTP_U creal
        @pre_filter
%if (@nn)
        Matrix<creal, @mp_cols, 1> dp;
        dp << g_x, Map<Matrix<creal,@ni,1> >(GET_U);
        @gen_mp
        int ret = nonlin::nonlin_solve(par);
        if (fnorm > g_fnorm) {
            g_fnorm = fnorm;
        }
        if (ret != 0) {
            return ret;
        }
        g_min = g_min.min(p_val.array());
        g_max = g_max.max(p_val.array());
        if (ii >= 0) {
            mi(ii) += GET_U[@ni];
        }
%end
        Matrix<creal, @m_cols, 1> d;
%if (@nn)
        d << g_x, Map<Matrix<creal,@ni,1> >(GET_U), mi@iblock;
%else
        d << g_x, Map<Matrix<creal,@ni,1> >(GET_U);
%end
%if (@nx)
        Matrix<creal, @nx, 1>& xn = g_x;
        @gen_xn
%end
        Map<Matrix<creal, @no, 1> > xo(o+@no*j);
        @gen_xo
#undef GET_U
#undef DTP_U
    }
    return 0;
}

extern "C" __attribute__ ((visibility ("default")))
int calc(creal *u, creal *x, creal *v, creal *x_new, creal *o, int *info, int *nfev, creal *fnorm) {
#define GET_U (u)
#define DTP_U creal
    @pre_filter
    int ret = 0;
    Matrix<creal, @nn, 1> mi;
%if (@nn)
    par.fnorm = fnorm;
    par.i = &mi;
    Matrix<creal, @mp_cols, 1> dp;
    dp << Map<Matrix<creal,@nx,1> >(x), Map<Matrix<creal,@ni,1> >(u);
    Matrix<creal, @nn, 1> mp;
    par.p = &mp;
    Map<Matrix<creal, @nn, 1> >Mv(v);
    par.v = &Mv;
    Array<creal, @nni, 1> p_val;
    par.p_val = &p_val;
    @gen_mp
    ret = nonlin::nonlin(par);
%else
    *info = 1;
    *nfev = 0;
    *fnorm = 0;
%end
    Matrix<creal, @m_cols, 1> d;
    d << Map<Matrix<creal,@nx,1> >(x), Map<Matrix<creal,@ni,1> >(u), mi@iblock;
    Map<Matrix<creal, @nx, 1> > xn(x_new);
    @gen_xn
    Map<Matrix<creal, @no, 1> > xo(o);
    @gen_xo
    return ret;
#undef GET_U
#undef DTP_U
}
%end

%if (@resample)
#include <zita-resampler/resampler.h>

class FixedRateResampler {
private:
    Resampler r_up, r_down;
    int inputRate, outputRate;
    int last_in_count;
public:
    int setup(int _inputRate, int _outputRate);
    int up(int count, float *input, float *output);
    void down(float *input, float *output);
    int max_out_count(int in_count) {
        return static_cast<int>(ceil((in_count*static_cast<double>(outputRate))/inputRate)); }
};

int FixedRateResampler::setup(int _inputRate, int _outputRate)
{
    const int qual = 16; // resulting in a total delay of 2*qual (0.7ms @44100)
    inputRate = _inputRate;
    outputRate = _outputRate;
    if (inputRate == outputRate) {
        return 0;
    }
    // upsampler
    int ret = r_up.setup(inputRate, outputRate, 1, qual);
    if (ret) {
        return ret;
    }
    // k == filtlen() == 2 * qual
    // pre-fill with k-1 zeros
    r_up.inp_count = r_up.filtlen() - 1;
    r_up.out_count = 1;
    r_up.inp_data = r_up.out_data = 0;
    r_up.process();
    // downsampler
    ret = r_down.setup(outputRate, inputRate, 1, qual);
    if (ret) {
        return ret;
    }
    // k == filtlen() == 2 * qual * fact
    // pre-fill with k-2 zeros
    r_down.inp_count = r_down.filtlen() - 2;
    r_down.out_count = 1;
    r_down.inp_data = r_down.out_data = 0;
    r_down.process();
    return 0;
}

int FixedRateResampler::up(int count, float *input, float *output)
{
    if (inputRate == outputRate) {
        memcpy(output, input, count*sizeof(float));
        r_down.out_count = count;
        return count;
    }
    r_up.inp_count = count;
    r_down.out_count = count+1; // +1 == trick to drain input
    r_up.inp_data = input;
    int m = max_out_count(count);
    r_up.out_count = m;
    r_up.out_data = output;
    r_up.process();
    assert(r_up.inp_count == 0);
    assert(r_up.out_count <= 1);
    r_down.inp_count = m - r_up.out_count;
    return r_down.inp_count;
}

void FixedRateResampler::down(float *input, float *output)
{
    if (inputRate == outputRate) {
        memcpy(output, input, r_down.out_count*sizeof(float));
        return;
    }
    r_down.inp_data = input;
    r_down.out_data = output;
    r_down.process();
    assert(r_down.inp_count == 0);
    assert(r_down.out_count == 1);
}

FixedRateResampler smp;
%end

class DKPlugin: public PluginDef {
public:
    float pots[@npl+@add_npl];
private:
    creal pots_last[@npl+@add_npl];
    Matrix<creal, @nx, 1> x_last;
    @DKPlugin_fields
public:
    EIGEN_MAKE_ALIGNED_OPERATOR_NEW;
    DKPlugin();
    static void init(unsigned int samplingFreq, PluginDef *plugin);
    static void process(int count, float *input, float *output, PluginDef *plugin);
    static int registerparam(const ParamReg& reg);
    static int uiloader(const UiBuilder& builder, int form);
    static void del_instance(PluginDef *plugin);
};

DKPlugin::DKPlugin():
    PluginDef(), pots(), pots_last(), x_last()@DKPlugin_init {
    version = PLUGINDEF_VERSION;
    flags = 0;
    id = @plugindef.s_id;
    name = @plugindef.l_name;
    groups = 0;
    description = @plugindef.l_description;
    category = @plugindef.l_category;
    shortname = @plugindef.l_shortname;
    mono_audio = process;
    set_samplerate = init;
    register_params = registerparam;
    load_ui = uiloader;
    delete_instance = del_instance;
    for (int i = 0; i < @nx; i++) {
        x_last(i) = x0_data[i];
    }
}

#define PARAM(p) ("@id" "." p)

int DKPlugin::registerparam(const ParamReg& reg) {
    %if (@regs)
        DKPlugin& self = *static_cast<DKPlugin*>(reg.plugin);
    %end
    %for @r in @regs:
        reg.registerVar(PARAM("@r.id"), N_("@r.name"), "S", N_("@r.desc"), &self.pots[@r.varidx], 0.5, 0, 1, 0.01);
    %end
    return 0;
}

void DKPlugin::init(unsigned int samplingFreq, PluginDef *plugin) {
%if (@resample)
    smp.setup(samplingFreq, @fs);
%end
%if (@filter_init)
    DKPlugin& self = *static_cast<DKPlugin*>(plugin);
    @filter_init
%end
}

void DKPlugin::process(int n, float *u, float *o, PluginDef *plugin) {
    DKPlugin& self = *static_cast<DKPlugin*>(plugin);
%if (@npl || @add_npl)
    creal t[@npl+@add_npl];
    @calc_pots
%end
    @process_add
// start copied and modified code
    Matrix<creal, @nn, 1> mi;
%if (@nn)
%%    g_min = Matrix<creal, @nni, 1>::Constant(HUGE_VAL);
%%    g_max = Matrix<creal, @nni, 1>::Constant(-HUGE_VAL);
%%    g_fnorm = 0;
    creal fnorm;
    Matrix<creal, @nn, 1> mp;
    Array<creal, @nni, 1> p_val;
    nonlin_param par@nonlin_mat_list_calc;
%end
%if (@resample)
    float buf[smp.max_out_count(n)];
    n = smp.up(n, u, buf);
#define GET_U (buf+j*@ni)
%else
#define GET_U (u+j*@ni)
%end
    for (int j = 0; j < n; j++) {
#define DTP_U float
        @pre_filter
%if (@npl)
        for (int k = 0; k < @npl; k++) {
            self.pots_last[k] = @timecst * t[k] + (1-@timecst) * self.pots_last[k];
        }
        calc_inv_update(self.pots_last, par);
%end
%if (@nn)
        Matrix<creal, @mp_cols, 1> dp;
        dp << self.x_last, Map<Matrix<float,@ni,1> >(GET_U).cast<creal>();
        @gen_mp
        nonlin::nonlin(par);
%%        int ret = nonlin::nonlin_solve(par);
%%        if (fnorm > g_fnorm) {
%%            g_fnorm = fnorm;
%%        }
%%        if (ret != 0) {
%%            return;
%%        }
%end
        Matrix<creal, @m_cols, 1> d;
        d << self.x_last, Map<Matrix<float,@ni,1> >(GET_U).cast<creal>(), mi@iblock;
%if (@nx)
        Matrix<creal, @nx, 1>& xn = self.x_last;
        @gen_xn
%end
%if (@resample)
        Map<Matrix<float, @no, 1> > xo(buf+@no*j);
%else
        Map<Matrix<float, @no, 1> > xo(o+@no*j);
%end
        @gen_xo_float
        @post_filter
#undef GET_U
#undef DTP_U
    }
%if (@resample)
    smp.down(buf, o);
%end
    @post_process
// end copied code
}

int DKPlugin::uiloader(const UiBuilder& b, int form) {
    if (!(form & UI_FORM_STACK)) {
        return -1;
    }
    %parse ("module_ui_template")
    return 0;
}

void DKPlugin::del_instance(PluginDef *p)
{
    delete static_cast<DKPlugin*>(p);
}

%if (@dev_interface)
extern "C" __attribute__ ((visibility ("default")))
int get_gx_plugin(unsigned int idx, PluginDef **pplugin)
{
    const int count = 1;
    if (!pplugin) {
        return count;
    }
    switch (idx) {
    case 0: *pplugin = new DKPlugin; return count;
    default: *pplugin = 0; return -1;
    }
}
%else
namespace pluginlib { namespace @plugindef.namespace {
PluginDef *plugin() {
    return new DKPlugin;
}
}}
%end

%if (@{plugindef.lv2_plugin_type})
%parse ("lv2_interface")
%end
""")

lv2_manifest = Template("""
\@prefix lv2:  <http://lv2plug.in/ns/lv2core#> .
\@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .

<http://guitarix.sourceforge.net/plugins/@id#@{plugindef.lv2_versioned_id}>
    a lv2:Plugin ;
    lv2:binary <@{plugindef.lv2_versioned_id}.so>  ;
    rdfs:seeAlso <@{plugindef.lv2_versioned_id}.ttl> .
""")

lv2_ttl = Template("""
\@prefix doap: <http://usefulinc.com/ns/doap#> .
\@prefix foaf: <http://xmlns.com/foaf/0.1/> .
\@prefix lv2: <http://lv2plug.in/ns/lv2core#> .
\@prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .
\@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .
\@prefix guiext: <http://lv2plug.in/ns/extensions/ui#>.

<http://guitarix.sourceforge.net#devel>
    a foaf:Group ;
    foaf:name "Guitarix team" ;
    foaf:mbox <mailto:guitarix-developer\@lists.sourceforge.net> ;
    rdfs:seeAlso <http://guitarix.sourceforge.net> .

<http://guitarix.sourceforge.net/plugins/@id>
    a doap:Project ;
    doap:maintainer <http://guitarix.sourceforge.net#devel> ;
    doap:name "@{plugindef.name}" .

<http://guitarix.sourceforge.net/plugins/@id#@{plugindef.lv2_versioned_id}>
    a lv2:Plugin ,
        lv2:@{plugindef.lv2_plugin_type} ;
    doap:maintainer <http://guitarix.sourceforge.net#devel> ;
    doap:name "@{plugindef.name}";
    doap:license <http://usefulinc.com/doap/licenses/gpl> ;
    lv2:project <http://guitarix.sourceforge.net/plugins/@id> ;
    lv2:optionalFeature lv2:hardRTCapable ;
    lv2:minorVersion @{plugindef.lv2_minor_version};
    lv2:microVersion @{plugindef.lv2_micro_version};

    lv2:port\
%for @p in @lv2_ports:\
 [
        a @{p.type_list} ;
        lv2:index @{p.index} ;
        lv2:symbol "@{p.symbol}" ;
        lv2:name "@{p.name}" ;
%if (@{p.control_index} >= 0)\
        lv2:default @{p.default} ;
        lv2:minimum @{p.minimum} ;
        lv2:maximum @{p.maximum} ;
%end
    ]%if(@velocityHasNext),%end\
%end.
""")

lv2_interface = Template("""
#include "lv2/lv2plug.in/ns/lv2core/lv2.h"

#define LV2_PLUGIN_URI "http://guitarix.sourceforge.net/plugins/@id#@{plugindef.lv2_versioned_id}"

typedef enum {
%for @p in @lv2_ports:\
    PORT_@{p.symbol} = @{p.index},
%end
} PortIndex;

class LV2_DKPlugin: public DKPlugin {
public:
    // Port buffers
    float* ports[@{lv2_ports.port_count()}];
public:
    LV2_DKPlugin(): DKPlugin() {}
};

static LV2_Handle
instantiate(const LV2_Descriptor*     descriptor,
            double                    rate,
            const char*               bundle_path,
            const LV2_Feature* const* features)
{
    LV2_DKPlugin *p = new LV2_DKPlugin;
    p->set_samplerate(rate, p);
    return static_cast<LV2_Handle>(p);
}

static void
connect_port(LV2_Handle instance,
             uint32_t   port,
             void*      data)
{
    static_cast<LV2_DKPlugin*>(instance)->ports[port] = static_cast<float*>(data);
}

static void
activate(LV2_Handle instance)
{
}

static void
run(LV2_Handle instance, uint32_t n_samples)
{
    LV2_DKPlugin* p = static_cast<LV2_DKPlugin*>(instance);

%for @p in @lv2_ports:\
%if (@{p.control_index} >= 0)\
    p->pots[@{p.control_index}] = *(p->ports[@{p.index}]);
%end
%end
%if (@ni == 1 && @no == 1)
    p->process(n_samples, p->ports[0], p->ports[1], p);
%else
    p->process(n_samples, p->ports[0], p->ports[1], p->ports[2], p->ports[3], p);
%end
}

static void
deactivate(LV2_Handle instance)
{
}

static void
cleanup(LV2_Handle instance)
{
    LV2_DKPlugin* p = static_cast<LV2_DKPlugin*>(instance);
    p->delete_instance(p);
}

static const void*
extension_data(const char* uri)
{
    return NULL;
}

static const LV2_Descriptor descriptor = {
    LV2_PLUGIN_URI,
    instantiate,
    connect_port,
    activate,
    run,
    deactivate,
    cleanup,
    extension_data
};

LV2_SYMBOL_EXPORT
extern "C" __attribute__ ((visibility ("default")))
const LV2_Descriptor*
lv2_descriptor(uint32_t index)
{
    switch (index) {
    case 0:
        return &descriptor;
    default:
        return NULL;
    }
}
""")

c_template_calc_nonlin = Template("""
%if (@dev_interface)
extern "C" __attribute__ ((visibility ("default")))
int calc_@{namespace}(int n, creal *p, creal *i, creal *v, int *info, int *nfev, creal *fnorm) {
    int ret = 0;
    Matrix<creal, @g_nn, 1> mp;
    Matrix<creal, @g_nn, 1> mi;
    Map<Matrix<creal, @g_nn, 1> >Mv(v);
    Array<creal, @g_nni, 1> p_val;
    nonlin_param par@nonlin_mat_list;
    for (int k = 0; k < n; k++) {
        calc_inv_update(p+k*(@nni+@npl), par);
        mp@pblockV << Map<Matrix<creal, @nni, 1> >(p+k*(@nni+@npl)+@npl);
        ret = @namespace::nonlin_solve(par);
        Map<Matrix<creal, @nno, 1> >(i+k*@nno) << mi@iblockV;
    }
    return ret;
}
%end
""")

c_template_nonlin_func_hybr = Template("""
static int fcn(void *p, int n, const creal *v, creal *fvec, int iflag ) {
    nonlin_param& par = *static_cast<nonlin_param *>(p);
    @expression
    return 0;
}
""")

c_template_nonlin_func_lm = Template("""
static int fcn(void *p, int m, int n, const creal *v, creal *fvec, int iflag ) {
    nonlin_param& par = *static_cast<nonlin_param *>(p);
    @expression
    return 0;
}
""")

c_template_nonlin_func_hybrCC = Template("""
typedef int root_fcn(void*p, const creal *v, creal *fvec, int iflag);
static int fcn(void *p, const creal *v, creal *fvec, int iflag) {
    nonlin_param& par = *static_cast<nonlin_param *>(p);
    @expression
    return 0;
}
""")

c_template_nonlin_solver_hybr = Template("""
static int nonlin(struct nonlin_param &par) {
    int maxfev, mode, nprint, ldfjac;
    creal xtol, epsfcn, factor;
    creal __attribute__((aligned(16))) fvec[@nn];
    creal __attribute__((aligned(16))) fjac[@nn*@nn];
    creal __attribute__((aligned(16))) qtf[@nn];
    creal __attribute__((aligned(16))) wa1[@nn];
    creal __attribute__((aligned(16))) wa2[@nn];
    creal __attribute__((aligned(16))) wa3[@nn];
    creal __attribute__((aligned(16))) wa4[@nn];
    creal diag[@nn] = {%for @j in @solver_diag:@j,%end};
    int ml, mu, lr;
    creal r[(@nn*(@nn+1))/2];
    lr = (@nn*(@nn+1))/2;
    ml = @nn-1; /* unbanded jacobian */
    mu = @nn-1; /* unbanded jacobian */
    ldfjac = @nn;

    /* parameter */
    xtol = @solver_xtol;
    maxfev = @solver_maxfev;
    epsfcn = 0.;
    //mode = 2;  /* explicit variable scaling with diag */
    mode = 1;  /* automatic variable scaling */
    factor = @solver_factor;
    nprint = 0;
    /**/

    @setup
    @store_p
    @p_transform
    *par.info = __cminpack_func__(hybrd)(
        fcn, &par, @nn, @var_v_ref, fvec, xtol, maxfev, ml, mu, epsfcn, diag,
        mode, factor, nprint, par.nfev, fjac, ldfjac, r, lr, qtf, wa1, wa2, wa3, wa4);
    *par.fnorm = __cminpack_func__(enorm)(@nn, fvec);
    @i_transform
    int ret = 0;
    if (*par.info != 1) {
        if (!(*par.info == 5 && *par.fnorm < 1e-20)) {
            ret = -1;
        }
    }
    @cleanup
    return ret;
}
""")

c_template_nonlin_solver_lm = Template("""
static int nonlin(struct nonlin_param &par) {
    int maxfev, mode, nprint, ldfjac;
    creal xtol, epsfcn, factor;
    creal __attribute__((aligned(16))) fvec[@nn];
    creal __attribute__((aligned(16))) fjac[@nn*@nn];
    creal __attribute__((aligned(16))) qtf[@nn];
    creal __attribute__((aligned(16))) wa1[@nn];
    creal __attribute__((aligned(16))) wa2[@nn];
    creal __attribute__((aligned(16))) wa3[@nn];
    creal __attribute__((aligned(16))) wa4[@nn];
    creal diag[@nn] = {%for @j in @solver_diag:@j,%end};
    int __attribute__((aligned(16))) ipvt[@nn];
    creal ftol, gtol;
    ftol = sqrt(__cminpack_func__(dpmpar)(1)); // parameter
    gtol = 0.; // parameter
    ldfjac = @nn;

    /* parameter */
    xtol = @solver_xtol;
    maxfev = @solver_maxfev;
    epsfcn = 0.;
    //mode = 2;  /* explicit variable scaling with diag */
    mode = 1;  /* automatic variable scaling */
    factor = @solver_factor;
    nprint = 0;
    /**/

    @setup
    @store_p
    @p_transform

    *par.info = __cminpack_func__(lmdif)(
        fcn, &par, @nn, @nn, @var_v_ref, fvec, ftol, xtol, gtol, maxfev, epsfcn, diag,
        mode, factor, nprint, par.nfev, fjac, ldfjac, ipvt, qtf, wa1, wa2, wa3, wa4);
    *par.fnorm = __cminpack_func__(enorm)(@nn, fvec);
    @i_transform
    @cleanup
    return (*par.info < 1 || *par.info > 4) ? -1 : 0;
}
""")

c_template_nonlin_solver_hybrCC = Template("""
%include ("hybrd.cc")

static int nonlin(struct nonlin_param &par) {
    int maxfev, mode, nprint, ldfjac;
    creal xtol, epsfcn, factor;
    creal __attribute__((aligned(16))) fvec[@nn];
    creal __attribute__((aligned(16))) fjac[@nn*@nn];
    creal __attribute__((aligned(16))) qtf[@nn];
    creal __attribute__((aligned(16))) wa1[@nn];
    creal __attribute__((aligned(16))) wa2[@nn];
    creal __attribute__((aligned(16))) wa3[@nn];
    creal __attribute__((aligned(16))) wa4[@nn];
    creal diag[@nn] = {%for @j in @solver_diag:@j,%end};
    int ml, mu, lr;
    creal r[(@nn*(@nn+1))/2];
    lr = (@nn*(@nn+1))/2;
    ml = @nn-1; /* unbanded jacobian */
    mu = @nn-1; /* unbanded jacobian */
    ldfjac = @nn;

    /* parameter */
    xtol = @solver_xtol;
    maxfev = @solver_maxfev;
    epsfcn = 0.;
    //mode = 2;  /* explicit variable scaling with diag */
    mode = 1;  /* automatic variable scaling */
    factor = @solver_factor;
    nprint = 0;
    /**/

    @setup
    @store_p
    @p_transform

    *par.info = hybrdX<@nn>(fcn, &par, @var_v_ref, fvec, xtol, maxfev, ml, mu, epsfcn,
                          diag, mode, factor, nprint, par.nfev, fjac, ldfjac, r, lr,
                          qtf, wa1, wa2, wa3, wa4);
    *par.fnorm = enorm<@nn>(fvec);
    @i_transform
    int ret = 0;
    if (*par.info != 1) {
        if (!(*par.info == 5 && *par.fnorm < 1e-20)) {
            ret = -1;
        }
    }
    @cleanup
    return ret;
}
""")

c_template_nonlin_chained = Template("""
namespace @namespace {
@global_data_def

static int nonlin(struct nonlin_param &par) {
    @chained_code
}

%if (@dev_interface)
static inline int nonlin_solve(nonlin_param& par) {
    return nonlin(par);
}
%end
} // end namespace @namespace
%if (@extern_nonlin)
%parse ("c_template_calc_nonlin")
%end
""")

c_template_nonlin_solver = Template("""
namespace @namespace {
@global_data_def

@fcn_def

@nonlin_def

%if (@dev_interface)
static Matrix<creal, @nni, 1> last_good;
static Matrix<creal, @nn, 1> last_v0;

int nonlin_homotopy(nonlin_param& par, creal f) {
    Matrix<creal, @nni, 1> end = @par_p;
    @par_p = last_good + (@par_p - last_good) * f;
    @par_v = last_v0;
    int ret = nonlin(par);
    @par_p = end;
    if (ret != 0) {
        return ret;
    }
    last_v0 = @par_v;
    return 0;
}

static inline int nonlin_solve(nonlin_param& par) {
    @v0_guess
    int ret = nonlin(par);
    if (ret != 0) {
        std::list<creal> points;
        points.push_back(0);
        points.push_back(0.5);
        points.push_back(1);
        for (int j = 0; j < @solver_max_homotopy_iter; j++) {
            std::list<creal>::iterator it = points.begin();
            ++it;
            ret = nonlin_homotopy(par, *it);
            if (ret != 0) {
                points.insert(it, (*points.begin()+*it)/2);
                continue;
            }
            if (points.size() == 2) {
                break;
            }
            points.erase(points.begin());
        }
        if (ret != 0) {
            return ret;
        }
    }
    last_good = @par_p;
    last_v0 = @par_v;
    return 0;
}
%end
} // end namespace @namespace

%if (@extern_nonlin)
%parse ("c_template_calc_nonlin")
%end
""")

c_template_table = Template("""
namespace @namespace {
static inline int nonlin(nonlin_param& par) {
    @call
    return 0;
}

%if (@dev_interface)
static inline int nonlin_solve(nonlin_param& par) {
    return nonlin(par);
}
%end

%parse ("c_template_calc_nonlin")
} // end @namespace
""")

#
#-mavx -ffast-math
#
# ffast-math seems to trigger a compiler error on gcc 4.7.3
# according to documentation it expands into list of settings, including
# funsafe-math-optimizations, which seems to be responsible for the bug.
# But funsafe-math-optimizations in turn just expands into another list of
# settings. When using the expanded list of settings the bug is gone (?!).
#
# maybe -mavx is the culprit (removed for now)??
#
build_script_template = Template("""\
#! /bin/bash
cd `dirname $0`
fname="${1-@sourcename}"
lname="${2-@soname}"
lname_="${2-@soname_}"
mo="-fwrapv -ffast-math -msse2"
#mo="-fwrapv -fno-math-errno -ffinite-math-only -fno-rounding-math"
#mo="$mo -fno-signaling-nans -fcx-limited-range -fno-signed-zeros"
#mo="$mo -fno-trapping-math -fassociative-math -freciprocal-math"
: ${CC:=c++}
%if (@debug)
dbg="-g -O2"
%else
dbg="-O2 -Wl,--strip-all -DNDEBUG"
%end
co="-shared -fPIC -Wall -fvisibility=hidden"
lo="-Wl,-O1,-Bsymbolic-functions,-z,relro"
libs="@libraries"
inc="@includes"
def="@defines"
opt="-fno-stack-protector"
set -e
%if (@optimize)
$CC $co $lo $mo $opt -fprofile-arcs $dbg $CFLAGS $def $inc "$fname" -o "$lname_" $libs
python t.py
$CC $co $lo $mo $opt -fprofile-use $dbg $CFLAGS $def $inc "$fname" -o "$lname" $libs
%else
$CC $co $lo $mo $opt $dbg $CFLAGS $def $inc "$fname" -o "$lname" $libs
%end
""")

faust_filter_template = Template("""\
%if (@build_script)\
// generated by @build_script
%else\
// generated automatically
%end
// DO NOT MODIFY!
declare id "@plugindef.id";
declare name "@plugindef.name";
declare category "@plugindef.category";
%if (@plugindef.shortname)\
declare shortname "@plugindef.shortname";
%end
%if (@plugindef.description)\
declare description "@plugindef.description";
%end

import("filter.lib");

process = pre : iir((@b_list),(@a_list)) with {
    LogPot(a, x) = if(a, (exp(a * x) - 1) / (exp(a) - 1), x);
    Inverted(b, x) = if(b, 1 - x, x);
    s = 0.993;
    fs = float(SR);
    pre = @pre_filter;

    %for @sl in @sliders:
        %if (@sl.loga)
        @sl.id = vslider("@sl.id[name:@sl.name]", 0.5, 0, 1, 0.01) : Inverted(@sl.inv) : LogPot(@sl.loga) : smooth(s);
        %else
        @sl.id = vslider("@sl.id[name:@sl.name]", 0.5, 0, 1, 0.01) : Inverted(@sl.inv) : smooth(s);
        %end
    %end

    @coeffs
};
""")

module_ui_template = Template("""
b.openHorizontalhideBox("");
%if (@have_master_slider)
    b.create_master_slider(PARAM("@master_slider_id"), "@master_slider_id");
%end
b.closeBox();
b.openHorizontalBox("");
%for @k in @knob_ids:
    b.create_small_rackknobr(PARAM("@k"), "@k");
%end
b.closeBox();
""")

faust_filter_dry_wet_template = Template("""\
%if (@build_script)\
// generated by @build_script
%else\
// generated automatically
%end
// DO NOT MODIFY!
declare id "@plugindef.id";
declare name "@plugindef.name";
declare category "@plugindef.category";
%if (@plugindef.shortname)\
declare shortname "@plugindef.shortname";
%end
%if (@plugindef.description)\
declare description "@plugindef.description";
%end

import("filter.lib");

process = pre : _<:*(dry),(*(wet) : iir((@b_list),(@a_list))):>_ with {
    LogPot(a, x) = if(a, (exp(a * x) - 1) / (exp(a) - 1), x);
    Inverted(b, x) = if(b, 1 - x, x);
    s = 0.993;
    fs = float(SR);
    pre = @pre_filter;
    wet = vslider("wet_dry[name:wet/dry][tooltip:percentage of processed signal in output signal]",  100, 0, 100, 1) : /(100);
    dry = 1 - wet;

    %for @sl in @sliders:
        @sl.id = vslider("@sl.id[name:@sl.name]", 0.5, 0, 1, 0.01) : Inverted(@sl.inv) : LogPot(@sl.loga) : smooth(s);
    %end

    @coeffs
};
""")

module_ui_dry_wet_template = Template("""
b.openHorizontalhideBox("");
%if (@have_master_slider)
    b.create_master_slider(PARAM("@master_slider_id"), "@master_slider_id");
%end
b.closeBox();
b.openHorizontalBox("");
%for @k in @knob_ids:
    b.create_small_rackknobr(PARAM("@k"), "@k");
%end
    b.create_small_rackknobr(PARAM("wet_dry"), "dry/wet");
b.closeBox();
""")



from pylab import *
from scipy.signal import freqz
import pluginloader

fs = 44100
dsp = pluginloader.Plugin("./example_plugin.so")
dsp.init(fs)
print "default level:", dsp["dsp.level"]
dsp["dsp.level"] = 0.5
impulse = zeros(5, dtype=float32)
impulse[0] = 1.0
w, h = freqz(dsp.compute(impulse))
# skip frequency 0 (h[0] == 0)
w = w[1:]
# scale 0..2*pi to 0 .. fs
w = w/(2*pi)*fs
# dB of absolute value
h = 20*log10(abs(h[1:]))
plot(w, h)
show()

## execution time:

inp = array(np.random.uniform(0, 1, 1000), float32)
n = 100
tm = zeros(n)
for i in range(n):
    dsp.compute(inp)
    tm[i] = dsp.nanosec_per_sample
print "%.2f +/- %.3f" % (mean(tm), std(tm))



#! /usr/bin/env python
# encoding: utf-8

import Task
from TaskGen import extension
bison='${BISON} ${BISONFLAGS} ${SRC[0].abspath()} -o ${TGT[0].name}'
cls=Task.simple_task_type('bison',bison,'GREEN',ext_in='.yc .y .yy',ext_out='.c .cxx .h .l',shell=False)
def big_bison(self,node):
	has_h='-d'in self.env['BISONFLAGS']
	outs=[]
	if node.name.endswith('.yc'):
		outs.append(node.change_ext('.tab.cc'))
		if has_h:
			outs.append(node.change_ext('.tab.hh'))
	else:
		outs.append(node.change_ext('.tab.c'))
		if has_h:
			outs.append(node.change_ext('.tab.h'))
	tsk=self.create_task('bison',node,outs)
	tsk.cwd=node.bld_dir(tsk.env)
	self.allnodes.append(outs[0])
def detect(conf):
	bison=conf.find_program('bison',var='BISON',mandatory=True)
	conf.env['BISONFLAGS']='-d'

extension(['.y','.yc','.yy'])(big_bison)
